-------------------------------------------------------------
-- MSS copyright 2015-2018
-- Filename:  COM1900.VHD
-- Authors: 
--		Alain Zarembowitch / MSS
-- Version: 0
-- Last modified: 1/14/18

-- Inheritance: 	com-1905_004b 10/19/17
--
-- description:  VHDL code template for the COM-1900 platform. 
-- Including: gigabit Ethernet MAC, IP, TCP, UDP stack.
-- 
-- Usage: keep the entire code and add your application where the TODO markers are.
-- Set the OPTION to 'A' for internal TCXO frequency reference or 'B' for external 10 MHz frequency reference.
-- In the latter case, the external 10 MHz signal must be present at power up.
--
--
-- IMPORTANT VIVADO SETTINGS: keep equivalent registers + disable resource sharing
--
---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
use work.com5402pkg.all;	-- defines global types, number of TCP streams, etc
Library UNISIM;
use UNISIM.vcomponents.all;

entity COM1900 is
	generic (
       OPTION: std_logic_vector(7 downto 0) := x"41";    -- 'A'    -- internal VCTCXO frequency reference 19.2MHz
       --OPTION: std_logic_vector(7 downto 0) := x"42";    -- 'B'    -- external 10 MHz frequency reference  
       REVISION: std_logic_vector(7 downto 0) := x"34";    -- '4'
            --// FPGA option and revision (ASCII format '0' = x30, 'A' = x41)
            -- These two ASCII characters appear in the Comblock Control Center.
            -- Helpful in identifying which firmware version is currently running in the FPGA. 
            -- Select internal vs external frequency reference option. 
            -- REMINDER: do change the .xdc file according to the option selection -A or -B
		SIMULATION: std_logic := '0'
			-- 1 during simulation, 0 for release	
	);
    Port ( 
		-- CLOCKS
		CLKREF_TCXO: in std_logic;
			-- 19.2 MHz TCXO frequency reference (also drive the ARM processor)
		CLKREF_EXT: in std_logic;
			-- 10 MHz external frequency reference
		CLKREF_OUT: out std_logic;
		CLK_LNB: out std_logic;
		CLK_TX_OUT: out std_logic;
			-- 10 MHz frequency reference to external devices (LNB, transmitter, SMA)

		--// ADC
		-- AD9634-170, 12-bit (provision for 14-bit)
		-- 200 MHz IF undersampling, 160 MSamples/s
		ADC_CLK_OUT_LVDS: out std_logic_vector(1 downto 0);
			-- LVDS differential clock to ADC, (1=P, 0=N)
		--ADC_DCO_IN_LVDS: in std_logic_vector(1 downto 0);
			-- ADC clock from ADC, LVDS.   (1=P, 0=N)
		ADC_D_IN_LVDS: in std_logic_vector(13 downto 0);
			-- ADC samples, 2's complement format. LVDS/DDR. (odd=P,even=N)
		ADC_SPI: inout std_logic_vector(3 downto 1);
			-- 1.8V
		
		--// AD9707 14-bit precision 175 MS/s D/A Converters, 3.3V supply 
		-- FPGA - high-speed DAC interface
		DAC1_DOUT: out std_logic_vector(13 downto 0) := (others => '0');
		DAC2_DOUT: out std_logic_vector(13 downto 0) := (others => '0');
			-- 16-bit output samples (1 per DAC_CLKOUT_P period) .
		DAC_CLKOUT_P: out std_logic;
		DAC_CLKOUT_N: out std_logic;
			-- differential (not LVDS!) reference clock (delayed 1/4 CLK period to 
			-- meet the DAC tDBS and tDBH setup and hold requirements
		DAC_SPI: inout std_logic_vector(3 downto 1);	-- DAC controls. common to both DACs
		DAC1_SPI_CSB: out std_logic;	
		DAC2_SPI_CSB: out std_logic;
		RF_OUT_ENB: out std_logic;
			-- enable the mixer output. Active low. Use ARM firmware rev 1.6 or above

		 --// RF Synthesizers
		TX_SYNTH_CTRL: inout std_logic_vector(4 downto 0);
		RX_SYNTH_CTRL: inout std_logic_vector(4 downto 0);
		  -- clk,data,latch enable,lock detect,RF output enable

		--// AUX DACs for fast rx RF/IF gain control 
		AUX_DAC1_S: inout std_logic_vector(3 downto 1);
		AUX_DAC2_S: inout std_logic_vector(3 downto 1);
		
        --// AHA 4524 IC Turbo-product code encoder/decoder interface
        -- Legacy. Discontinued.
        T_RESET_N: out std_logic;
		
		--// DDR3 SDRAM. 1.5 V
		-- Inouts
		ddr3_dq                        : inout std_logic_vector(15 downto 0);
		ddr3_dqs_p                     : inout std_logic_vector(1 downto 0);
		ddr3_dqs_n                     : inout std_logic_vector(1 downto 0);

		-- Outputs
		ddr3_addr                      : out   std_logic_vector(12 downto 0);
		ddr3_ba                        : out   std_logic_vector(2 downto 0);
		ddr3_ras_n                     : out   std_logic;
		ddr3_cas_n                     : out   std_logic;
		ddr3_we_n                      : out   std_logic;
		ddr3_reset_n                   : out   std_logic;
		ddr3_ck_p                      : out   std_logic;
		ddr3_ck_n                      : out   std_logic;
		ddr3_cke                       : out   std_logic;
		ddr3_cs_n                      : out   std_logic;
		ddr3_dm                        : out   std_logic_vector(1 downto 0);
		ddr3_odt                       : out   std_logic;

		--// 2 ETHERNET PHYs interface (MICREL KSZ9031)
		LAN1_TXC: out std_logic;
		LAN1_TX_CTL: out std_logic;
		LAN1_TXD: out std_logic_vector(3 downto 0);
		LAN1_RXC: in std_logic;
		LAN1_RX_CTL: in std_logic;
		LAN1_RXD: in std_logic_vector(3 downto 0);
		LAN1_MCLK: out std_logic;
		LAN1_MDIO: inout std_logic;
		LAN1_INT_N: in std_logic;
		LAN1_RESET_N: out std_logic;
		LAN1_CLK125_NDO: in std_logic;

		LAN2_TXC: out std_logic;
		LAN2_TX_CTL: out std_logic;
		LAN2_TXD: out std_logic_vector(3 downto 0);
		LAN2_RXC: in std_logic;
		LAN2_RX_CTL: in std_logic;
		LAN2_RXD: in std_logic_vector(3 downto 0);
		LAN2_MCLK: out std_logic;
		LAN2_MDIO: inout std_logic;
		LAN2_INT_N: in std_logic;
		LAN2_RESET_N: out std_logic;

		--// Test points, LEDs
		TP: out std_logic_vector(12 downto 1);
      LED: out std_logic_vector(6 downto 2);
            -- front-panel LEDs. red led(1) is controlled directly by the ARM micro
        
		--// Monitoring & Control: microcontroller interface
		-- asynchronous bus (shared with NAND flash)
		-- set address at the rising edge of UC_WEB_IN when UC_CSIB_IN = '0' and UC_ALE_IN = '1'
		-- write data the rising edge of UC_WEB_IN when UC_CSIB_IN = '0'
		-- read data when  UC_CSIB_IN = '0' and UC_REB_IN = '0'
		UC_CSIB_IN: in std_logic;  -- chip select. active low.
		UC_ALE_IN: in std_logic;  -- address latch enable. active high.
		UC_REB_IN: in std_logic;  -- read enable #
		UC_WEB_IN: in std_logic;	-- write enable #
		UC_AD: inout std_logic_vector(7 downto 0)  -- shared address/data bus

		  
			  );
end entity;

architecture Behavioral of COM1900 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT CLKGEN7_MMCM_ADJ
	GENERIC (
		CLKFBOUT_MULT_F: real;
		CLKOUT0_DIVIDE_F: real;
		CLKIN1_PERIOD: real
    );    
    PORT(
		CLK_IN1 : IN std_logic;
		CLK_OUT1 : OUT std_logic;
		PHASE_SHIFT_VAL: in std_logic_vector(7 downto 0);
		PHASE_SHIFT_TRIGGER_TOGGLE: in std_logic;
		INPUT_CLK_STOPPED : OUT std_logic;
		LOCKED : OUT std_logic
        );
    END COMPONENT;
    
	COMPONENT CLKGEN7_MMCM_DYNAMIC is
	GENERIC (
      CLKIN1_PERIOD: real
    );    
	PORT(
      CLK_IN1           : in     std_logic;
      SYNC_RESET           : in     std_logic;
      M: in std_logic_vector(9 downto 0);   
      D: in std_logic_vector(6 downto 0);
      O: in std_logic_vector(10 downto 0);
      PROGRAM: in std_logic;
      CLK_OUT1          : out    std_logic;
      INPUT_CLK_STOPPED : out    std_logic;
      LOCKED            : out    std_logic
     );
	END COMPONENT;

--	COMPONENT BRAM_DP
--    GENERIC(
--        DATA_WIDTHA: integer;
--        ADDR_WIDTHA: integer;
--        DATA_WIDTHB: integer;
--        ADDR_WIDTHB: integer;
--		INFER: std_logic
--    );
--    PORT(
--        CLKA   : in  std_logic;
--        WEA    : in  std_logic;
--        ADDRA  : in  std_logic_vector(ADDR_WIDTHA-1 downto 0);
--        DIA   : in  std_logic_vector(DATA_WIDTHA-1 downto 0);
--        DOA  : out std_logic_vector(DATA_WIDTHA-1 downto 0);
--        CLKB   : in  std_logic;
--        WEB    : in  std_logic;
--        ADDRB  : in  std_logic_vector(ADDR_WIDTHB-1 downto 0);
--        DIB   : in  std_logic_vector(DATA_WIDTHB-1 downto 0);
--        DOB  : out std_logic_vector(DATA_WIDTHB-1 downto 0)
--        );
--    END COMPONENT;

	COMPONENT BRAM_DP2
	GENERIC(
		DATA_WIDTHA: integer;
		ADDR_WIDTHA: integer;
		DATA_WIDTHB: integer;
		ADDR_WIDTHB: integer
	);
	PORT(
		CLKA   : in  std_logic;
		CSA: in std_logic;	
		WEA    : in  std_logic;	
		OEA : in std_logic;	
		ADDRA  : in  std_logic_vector(ADDR_WIDTHA-1 downto 0);
		DIA   : in  std_logic_vector(DATA_WIDTHA-1 downto 0);
		DOA  : out std_logic_vector(DATA_WIDTHA-1 downto 0);
		CLKB   : in  std_logic;
		CSB: in std_logic;	-- chip select, active high
		WEB    : in  std_logic;	-- write enable, active high
		OEB : in std_logic;	-- output enable, active high
		ADDRB  : in  std_logic_vector(ADDR_WIDTHB-1 downto 0);
		DIB   : in  std_logic_vector(DATA_WIDTHB-1 downto 0);
		DOB  : out std_logic_vector(DATA_WIDTHB-1 downto 0)
		);
	END COMPONENT;

	COMPONENT TIMER_4US
	GENERIC (
		CLK_FREQUENCY: integer 
	);
	PORT(
		CLK : IN std_logic;          
		SYNC_RESET : IN std_logic;
		TICK_4US : OUT std_logic;
		TICK_100MS: out std_logic
		);
	END COMPONENT;

	 COMPONENT SPI
	 GENERIC(
		CONTROL_WIDTH: integer;
		NDIV: integer range 2 to 256
	 );
	 PORT(
			CLK : IN  std_logic;
			ASYNC_RESET : IN  std_logic;
			ADDR: in std_logic_vector((CONTROL_WIDTH-4) downto 0);
			DATA_OUT : IN  std_logic_vector(7 downto 0);
			WRITE_REQ : IN  std_logic;
			READ_REQ : IN  std_logic;
			DATA_IN : OUT  std_logic_vector(7 downto 0);
			DATA_IN_CLK : OUT  std_logic;
			BUSY : OUT  std_logic;
			CSB : OUT  std_logic;
			SCLK : OUT  std_logic;
			SDIO : INOUT  std_logic
		  );
	 END COMPONENT;

	COMPONENT CROSS_CLK_DOMAINS
	generic (
		NBITS: in integer
	);
	PORT(
		CLK_A : in std_logic;				-- 'from' clock domain
		CLK_B : in std_logic;				-- 'to' clock domain
		ASYNC_RESET: in std_logic;
		DATA_IN: in std_logic_vector((NBITS-1) downto 0);
		DATA_OUT: out std_logic_vector((NBITS-1) downto 0)
		);
	END COMPONENT;


	COMPONENT DAC_AD5621
	generic (
		CLK_FREQUENCY: integer 
	);
	PORT(
		CLK : IN std_logic;
		DATA_IN : IN std_logic_vector(11 downto 0);
		DATA_IN_CLK : IN std_logic;
		OP_MODE : IN std_logic_vector(1 downto 0);          
		DONE : OUT std_logic;
		SYNC_B : OUT std_logic;
		SCLK : OUT std_logic;
		SDIN : OUT std_logic
		);
	END COMPONENT;

	COMPONENT COM5401
	GENERIC (
		MII_SEL: std_logic;  
		PHY_ADDR: std_logic_vector(4 downto 0);
		CLK_FREQUENCY: integer; 
		PLATFORM: integer;
		TARGET_PHY: integer;
		INTERNAL_PHY_CLKS_DELAYS: std_logic;
		SIMULATION: std_logic
	);
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		MAC_TX_CONFIG : IN std_logic_vector(15 downto 0);
		MAC_RX_CONFIG : IN std_logic_vector(15 downto 0);
		MAC_ADDR : IN std_logic_vector(47 downto 0);
		PHY_CONFIG_CHANGE : IN std_logic;
		PHY_RESET : IN std_logic;
		SPEED : IN std_logic_vector(1 downto 0);
		DUPLEX : IN std_logic;
		TEST_MODE : IN std_logic_vector(1 downto 0);
		POWER_DOWN : IN std_logic;
		CLK_SKEW1: in std_logic_vector(15 downto 0);
		CLK_SKEW2: in std_logic_vector(15 downto 0);
		MAC_TX_DATA : IN std_logic_vector(7 downto 0);
		MAC_TX_DATA_VALID : IN std_logic;
		MAC_TX_EOF : IN std_logic;
		MAC_RX_CTS : IN std_logic;
		RXC : IN std_logic;
		RXD : IN std_logic_vector(3 downto 0);
		RX_CTL : IN std_logic;
		MII_TX_CLK : IN std_logic;
		GMII_MII_CRS : IN std_logic;
		GMII_MII_COL : IN std_logic;
		GMII_MII_RX_CLK : IN std_logic;
		GMII_MII_RXD : IN std_logic_vector(7 downto 0);
		GMII_MII_RX_DV : IN std_logic;
		GMII_MII_RX_ER : IN std_logic;    
		MDIO : INOUT std_logic;      
		MAC_TX_CTS : OUT std_logic;
		MAC_RX_DATA : OUT std_logic_vector(7 downto 0);
		MAC_RX_DATA_VALID : OUT std_logic;
		MAC_RX_SOF : OUT std_logic;
		MAC_RX_EOF : OUT std_logic;
		TXC : OUT std_logic;
		TXD : OUT std_logic_vector(3 downto 0);
		TX_CTL : OUT std_logic;
		RESET_N : OUT std_logic;
		MCLK : OUT std_logic;
		GMII_TX_CLK : OUT std_logic;
		GMII_MII_TXD : OUT std_logic_vector(7 downto 0);
		GMII_MII_TX_EN : OUT std_logic;
		GMII_MII_TX_ER : OUT std_logic;
		LINK_STATUS : OUT std_logic;
		SPEED_STATUS : OUT std_logic_vector(1 downto 0);
		DUPLEX_STATUS : OUT std_logic;
		PHY_ID: out std_logic_vector(15 downto 0);
		N_RX_FRAMES: out  std_logic_vector(15 downto 0);
		N_RX_BAD_CRCS: out  std_logic_vector(15 downto 0);
		N_RX_FRAMES_TOO_SHORT: out  std_logic_vector(15 downto 0);
		N_RX_FRAMES_TOO_LONG: out  std_logic_vector(15 downto 0);
		N_RX_WRONG_ADDR: out  std_logic_vector(15 downto 0);
		N_RX_LENGTH_ERRORS: out  std_logic_vector(15 downto 0);
		CS_CLKG: out std_logic;	-- synchronous clock for the comscope traces
		CS1: out std_logic_vector(7 downto 0);
		CS1_CLK: out std_logic;
		CS2: out std_logic_vector(7 downto 0);
		CS2_CLK: out std_logic;
		CT1: out std_logic;
		CT1_CLK: out std_logic;
		TP: out std_logic_vector(10 downto 1)
		);
	END COMPONENT;

	COMPONENT COM5402
	GENERIC (
		NUDPTX: integer range 0 to 1;
		NUDPRX: integer range 0 to 1;
		IGMP_EN: std_logic := '1';
		NTCPSTREAMS: integer range 0 to 255;  
		CLK_FREQUENCY: integer;
		TX_IDLE_TIMEOUT: integer range 0 to 50;	
		TCP_KEEPALIVE_PERIOD: integer := 60;
		SIMULATION: std_logic
	);	
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		MAC_ADDR : IN std_logic_vector(47 downto 0);
		IPv4_ADDR : IN std_logic_vector(31 downto 0);
		IPv6_ADDR : IN std_logic_vector(127 downto 0);
		MULTICAST_IP_ADDR: in std_logic_vector(31 downto 0);
		SUBNET_MASK: in std_logic_vector(31 downto 0);
		GATEWAY_IP_ADDR: in std_logic_vector(31 downto 0);
		CONNECTION_RESET: in std_logic_vector((NTCPSTREAMS-1) downto 0);
		TCP_KEEPALIVE_EN: in std_logic;
		MAC_TX_CTS : IN std_logic;
		MAC_RX_DATA : IN std_logic_vector(7 downto 0);
		MAC_RX_DATA_VALID : IN std_logic;
		MAC_RX_SOF : IN std_logic;
		MAC_RX_EOF : IN std_logic;          
		MAC_TX_DATA : OUT std_logic_vector(7 downto 0);
		MAC_TX_DATA_VALID : OUT std_logic;
		MAC_TX_SOF : OUT std_logic;
		MAC_TX_EOF : OUT std_logic;
		UDP_RX_DATA: out std_logic_vector(7 downto 0);
		UDP_RX_DATA_VALID: out std_logic;
		UDP_RX_SOF: out std_logic;	
		UDP_RX_EOF: out std_logic;	
		UDP_RX_DEST_PORT_NO_IN: in std_logic_vector(15 downto 0);
		CHECK_UDP_RX_DEST_PORT_NO: in std_logic;
		UDP_RX_DEST_PORT_NO_OUT: out std_logic_vector(15 downto 0);
		UDP_TX_DATA: in std_logic_vector(7 downto 0);
		UDP_TX_DATA_VALID: in std_logic;
		UDP_TX_SOF: in std_logic;	
		UDP_TX_EOF: in std_logic;	
		UDP_TX_CTS: out std_logic;	
		UDP_TX_ACK: out std_logic;	
		UDP_TX_NAK: out std_logic;	
		UDP_TX_DEST_IP_ADDR: in std_logic_vector(127 downto 0);
		UDP_TX_DEST_PORT_NO: in std_logic_vector(15 downto 0);
		UDP_TX_SOURCE_PORT_NO: in std_logic_vector(15 downto 0);
		TCP_RX_DATA: out SLV8xNTCPSTREAMStype;
		TCP_RX_DATA_VALID: out std_logic_vector((NTCPSTREAMS-1) downto 0);
		TCP_RX_RTS: out std_logic_vector((NTCPSTREAMS-1) downto 0);	-- Ready To Send
		TCP_RX_CTS: in std_logic_vector((NTCPSTREAMS-1) downto 0);	-- Clear To Send
		TCP_TX_DATA: in SLV8xNTCPSTREAMStype;
		TCP_TX_DATA_VALID: in std_logic_vector((NTCPSTREAMS-1) downto 0);
		TCP_TX_CTS: out std_logic_vector((NTCPSTREAMS-1) downto 0);	
		TCP_CONNECTED_FLAG: out std_logic_vector((NTCPSTREAMS-1) downto 0);
		CS1 : OUT std_logic_vector(7 downto 0);
		CS1_CLK : OUT std_logic;
		CS2 : OUT std_logic_vector(7 downto 0);
		CS2_CLK : OUT std_logic;
		TP : OUT std_logic_vector(10 downto 1)
		);
	END COMPONENT;

	COMPONENT ELASTIC_BUFFER_NRAMB
	GENERIC (
		NRAMB: integer;
		NBITS: integer;
		READ_AHEAD_1ADDRESS: std_logic
	);	
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		DATA_IN : IN std_logic_vector(8 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		SAMPLE_CLK_OUT_REQ : IN std_logic;          
		SAMPLE_CLK_IN_REQ : OUT std_logic;
		DATA_OUT : OUT std_logic_vector(8 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic;
		SAMPLE_CLK_OUT_E : OUT std_logic;
		BUFFER_EMPTY: out std_logic
		);
	END COMPONENT;

	COMPONENT DNA_ID
	GENERIC (
		CLK_FREQUENCY: integer
	);	
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;          
		ID_OUT : OUT std_logic_vector(56 downto 0) := (others => '0');
		ID_VALID_OUT : OUT std_logic;
		ID_SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT COMSCOPE2
	generic (
		TRACE1_ENABLED: std_logic;	
		TRACE2_ENABLED: std_logic;	
		TRACE3_ENABLED: std_logic;	
		TRACE4_ENABLED: std_logic;
		DATA_WIDTH1A: integer;
		ADDR_WIDTH1A: integer;
		ADDR_WIDTH1B: integer;
		DATA_WIDTH2A: integer;
		ADDR_WIDTH2A: integer;
		ADDR_WIDTH2B: integer;
		DATA_WIDTH3A: integer;
		ADDR_WIDTH3A: integer;
		ADDR_WIDTH3B: integer;
		DATA_WIDTH4A: integer;
		ADDR_WIDTH4A: integer;
		ADDR_WIDTH4B: integer
	);	
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		REG238 : IN std_logic_vector(7 downto 0);
		REG239 : IN std_logic_vector(7 downto 0);
		REG240 : IN std_logic_vector(7 downto 0);
		REG241 : IN std_logic_vector(7 downto 0);
		REG242 : IN std_logic_vector(7 downto 0);
		REG243 : IN std_logic_vector(7 downto 0);
		REG244 : IN std_logic_vector(7 downto 0);
		REG245 : IN std_logic_vector(7 downto 0);
		REG246 : IN std_logic_vector(7 downto 0);
		REG247 : IN std_logic_vector(7 downto 0);
		REG248 : IN std_logic_vector(7 downto 0);
		REG249 : IN std_logic_vector(7 downto 0);
		S1_1 : IN std_logic_vector(7 downto 0);
		S1_1_VALID : IN std_logic;
		S1_2 : IN std_logic_vector(7 downto 0);
		S1_2_VALID : IN std_logic;
		S1_3 : IN std_logic_vector(7 downto 0);
		S1_3_VALID : IN std_logic;
		S1_4 : IN std_logic_vector(7 downto 0);
		S1_4_VALID : IN std_logic;
		S2_1 : IN std_logic_vector(7 downto 0);
		S2_1_VALID : IN std_logic;
		S2_2 : IN std_logic_vector(7 downto 0);
		S2_2_VALID : IN std_logic;
		S2_3 : IN std_logic_vector(7 downto 0);
		S2_3_VALID : IN std_logic;
		S2_4 : IN std_logic_vector(7 downto 0);
		S2_4_VALID : IN std_logic;
		S3_1 : IN std_logic_vector(7 downto 0);
		S3_1_VALID : IN std_logic;
		S3_2 : IN std_logic_vector(7 downto 0);
		S3_2_VALID : IN std_logic;
		S3_3 : IN std_logic_vector(7 downto 0);
		S3_3_VALID : IN std_logic;
		S3_4 : IN std_logic_vector(7 downto 0);
		S3_4_VALID : IN std_logic;
		S4_1 : IN std_logic_vector(7 downto 0);
		S4_1_VALID : IN std_logic;
		S4_2 : IN std_logic_vector(7 downto 0);
		S4_2_VALID : IN std_logic;
		S4_3 : IN std_logic_vector(7 downto 0);
		S4_3_VALID : IN std_logic;
		S4_4 : IN std_logic_vector(7 downto 0);
		S4_4_VALID : IN std_logic;
		T16b_1 : IN std_logic_vector(15 downto 0);
		T16b_1_VALID : IN std_logic;
		T16b_2 : IN std_logic_vector(15 downto 0);
		T16b_2_VALID : IN std_logic;
		T16b_3 : IN std_logic_vector(15 downto 0);
		T16b_3_VALID : IN std_logic;
		T16b_4 : IN std_logic_vector(15 downto 0);
		T16b_4_VALID : IN std_logic;
		TRIGGER_REARM_TOGGLE : IN std_logic;
		FORCE_TRIGGER_TOGGLE : IN std_logic;
		SREG250_READ_TOGGLE : IN std_logic;
		START_CAPTURE_TOGGLE : IN std_logic;          
		SREG250 : OUT std_logic_vector(7 downto 0);
		SREG251 : OUT std_logic_vector(7 downto 0);
		TP : OUT std_logic_vector(16 downto 1)
		);
	END COMPONENT;

	component SIM2OUTFILE is
		 Generic (
			FILENAME: string
		 );
		 Port ( 
			--// CLK, RESET
			CLK: in std_logic;
				-- Reference clock
			DATA1: in std_logic_vector(11 downto 0);
			DATA2: in std_logic_vector(11 downto 0);
			DATA3: in std_logic_vector(11 downto 0);
			SAMPLE_CLK_IN: in std_logic
				-- DATA_IN is read at the rising edge of CLK when SAMPLE_CLK_IN = '1'

	);
	END COMPONENT;
	
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _X indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name

--// CLOCKS, RESETS
signal CLKGEN7A_INPUT_CLK_STOPPED: std_logic := '0';
signal CLKGEN7A_LOCKED: std_logic := '0';
signal CLKGEN7B_INPUT_CLK_STOPPED: std_logic := '0';
signal CLKGEN7B_LOCKED: std_logic := '0';
signal CLKGEN7C_INPUT_CLK_STOPPED: std_logic := '0';
signal CLKGEN7C_LOCKED: std_logic := '0';
signal CLKREF_TCXOg: std_logic := '0';
signal CLK10g: std_logic := '0';
signal CLK_P: std_logic := '0';
signal CLK_ADCg: std_logic := '0';
signal CLK_ADC_OUT: std_logic := '0';
signal ADC_CLK_IN: std_logic := '0';
signal CLK_RXg: std_logic := '0';
signal CLK_TXg: std_logic := '0';
signal CLK_TXg2: std_logic := '0';
signal CLK_TXg_FB: std_logic := '0';
signal CLK_TXg2_LOCKED: std_logic := '0';
signal READY_FLAG_CLK_P: std_logic := '0';
signal RESET_COUNTER: unsigned(23 downto 0) := x"000000";	-- initialize for simulations
signal ASYNC_RESET: std_logic := '0';
signal ASYNC_RESET_D: std_logic := '0';
signal ASYNC_RESET_D2: std_logic := '0';
signal SYNC_RESET_CLK_P: std_logic := '0';
signal ASYNC_RESET3_D: std_logic := '0';
signal ASYNC_RESET3_D2: std_logic := '0';
signal SYNC_RESET_CLK_TXgA: std_logic := '0';
signal SYNC_RESET_CLK_TXg: std_logic := '0';
signal ASYNC_RESET2_D: std_logic := '0';
signal ASYNC_RESET2_D2: std_logic := '0';
signal SYNC_RESET_CLK_RXg: std_logic := '0';
signal ASYNC_RESET4_D: std_logic := '0';
signal ASYNC_RESET4_D2: std_logic := '0';
signal SYNC_RESET_CLK48g: std_logic := '0';

signal CLK_TX_GEN_M: std_logic_vector(9 downto 0) := (others => '0');
signal CLK_TX_GEN_D: std_logic_vector(6 downto 0) := (others => '0');
signal CLK_TX_GEN_O: std_logic_vector(10 downto 0) := (others => '0');
signal CLK_TX_GEN_LOCKED: std_logic := '0';
signal CLK_TX_GEN_INPUT_CLK_STOPPED: std_logic := '0';
signal CLK_TX_GEN_SYNC_RESET: std_logic := '0';
signal CLK_TX_GEN_PROGRAM: std_logic := '0';
signal RESET_STATE: integer range 0 to 3 := 0;

signal TICK_1S: std_logic := '0';
signal TICK_4US: std_logic := '0';
signal TICK_100MS_rt: std_logic := '0';
signal TICK_100MS_CNTR: unsigned(3 downto 0) := (others => '0');

--// AUX DACs (RF/IF AGC) ---------------------------------------------------------
signal RF_AGC_DAC: std_logic_vector(11 downto 0) := (others => '0');
signal RF_AGC_DAC_SAMPLE_CLK: std_logic := '0';
signal IF_AGC_DAC: std_logic_vector(11 downto 0) := (others => '0');
signal IF_AGC_DAC_SAMPLE_CLK: std_logic := '0';

--// ADC -------------------------------------
signal ADC_D_IN_DDR: std_logic_vector(6 downto 0) := (others => '0');
signal ADC_D_IN: std_logic_vector(13 downto 0) := (others => '0'); 
signal ADC_DATA_IN: std_logic_vector(13 downto 0) := (others => '0');
signal ADC_SPI_STATE: integer range 0 to 63 := 0;
signal ADC_SPI_WRITE_REQ: std_logic := '0';
signal ADC_SPI_READ_REQ: std_logic := '0';
signal ADC_SPI_BUSY: std_logic := '0';
signal ADC_SPI_ADDR: std_logic_vector(12 downto 0) := (others => '0');
signal ADC_SPI_DATA_OUT: std_logic_vector(7 downto 0) := (others => '0');
signal ADC_SPI_DATA_IN: std_logic_vector(7 downto 0) := (others => '0');
signal ADC_SPI_DATA_IN_CLK: std_logic := '0';

--// DUAL 175MS/S DAC ------------------
signal TEST_RAMP1: signed(13 downto 0) := (others => '0');
signal TEST_RAMP2: signed(13 downto 0) := (others => '0');

--// LOOP BACK -----------------------------------------------
signal LOOPBACK_DATA1_I: std_logic_vector(13 downto 0) := (others => '0');
signal LOOPBACK_DATA1_Q: std_logic_vector(13 downto 0) := (others => '0');
signal LOOPBACK_TEST_MODE: std_logic := '0';

--// RECEIVER INPUT SELECTION -------------------------------------
signal RX_DATA1_I: std_logic_vector(13 downto 0) := (others => '0');
signal RX_DATA1_Q: std_logic_vector(13 downto 0) := (others => '0');


--//-- 10/100/1000 Mbps Ethernet -------
signal CLK_SKEW1: std_logic_vector(15 downto 0) := (others => '0');
signal CLK_SKEW2: std_logic_vector(15 downto 0) := (others => '0');
--signal LAN1_CLK125G: std_logic:= '0';
signal LAN1_MAC_ADDR: std_logic_vector(47 downto 0) := (others => '0');
signal COM5401_TP: std_logic_vector(10 downto 1) := (others => '0');
signal LAN1_PHY_ID: std_logic_vector(15 downto 0) := (others => '0');
signal LAN1_MAC_RX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN1_MAC_RX_DATA_VALID: std_logic:= '0';
signal LAN1_MAC_RX_SOF: std_logic:= '0';
signal LAN1_MAC_RX_EOF: std_logic:= '0';
signal LAN1_MAC_TX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN1_MAC_TX_DATA_VALID: std_logic:= '0';
signal LAN1_MAC_TX_SOF: std_logic:= '0';
signal LAN1_MAC_TX_EOF: std_logic:= '0';
signal LAN1_MAC_TX_CTS: std_logic:= '0';
signal LAN1_LINK_STATUS: std_logic:= '0';
signal LAN1_SPEED_STATUS: std_logic_vector(1 downto 0) := (others => '0');
signal LAN1_DUPLEX_STATUS: std_logic:= '0';
signal LAN1_N_RX_BAD_CRCS: std_logic_vector(15 downto 0) := (others => '0');

signal LAN2_MAC_ADDR: std_logic_vector(47 downto 0) := (others => '0');
signal LAN2_PHY_ID: std_logic_vector(15 downto 0) := (others => '0');
signal LAN2_MAC_RX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN2_MAC_RX_DATA_VALID: std_logic:= '0';
signal LAN2_MAC_RX_SOF: std_logic:= '0';
signal LAN2_MAC_RX_EOF: std_logic:= '0';
signal LAN2_MAC_TX_DATA: std_logic_vector(7 downto 0) := (others => '0');
signal LAN2_MAC_TX_DATA_VALID: std_logic:= '0';
signal LAN2_MAC_TX_EOF: std_logic:= '0';
signal LAN2_MAC_TX_CTS: std_logic:= '0';
signal LAN2_LINK_STATUS: std_logic:= '0';
signal LAN2_SPEED_STATUS: std_logic_vector(1 downto 0) := (others => '0');
signal LAN2_DUPLEX_STATUS: std_logic:= '0';
signal LAN2_N_RX_BAD_CRCS: std_logic_vector(15 downto 0) := (others => '0');

-- REMOTE M&C  TCP SERVER
constant NTCPSTREAMS: integer := 1;  -- number of concurrent TCP streams handled by this component
signal COM5402_TP: std_logic_vector(10 downto 1) := (others => '0');
signal IP1_ADDRv4: std_logic_vector(31 downto 0) := (others => '0');	
signal IP1_SUBNETMASK: std_logic_vector(31 downto 0) := (others => '0');	
signal LAN1_GATEWAY: std_logic_vector(31 downto 0) := (others => '0');	
signal LAN1_TCP_RX_DATA: SLV8xNTCPSTREAMStype;
signal LAN1_TCP_RX_DATA_VALID: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_RX_RTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_RX_CTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_TX_DATA: SLV8xNTCPSTREAMStype;
signal LAN1_TCP_TX_DATA_VALID: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal LAN1_TCP_TX_CTS: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');
signal TCP_CONNECTED_FLAG: std_logic_vector((NTCPSTREAMS-1) downto 0) := (others => '0');

--// LAN TCP-IP Interface for M&C (port 1028)
signal RX_LAN: std_logic_vector(7 downto 0) := (others => '0'); 
signal RX_LAN_SAMPLE_CLK_REQ: std_logic := '0';
signal RX_LAN_BUFFER_EMPTY: std_logic  := '0';
signal RX_LAN_BUFFER_EMPTY_D: std_logic  := '0';
signal RX_LAN_BUFFER_EMPTY_D2: std_logic  := '0';
signal TX_LAN_TOGGLE: std_logic  := '0';
signal TX_LAN_TOGGLE_D: std_logic  := '0';
signal TX_LAN_TOGGLE_D2: std_logic  := '0';
signal LAN1_TCP_RX_DATA0_X: std_logic_vector(8 downto 0) := (others => '0'); 
signal LAN_RX_DATA1: std_logic_vector(7 downto 0) := (others => '0');
signal LAN_RX_DATA1_X: std_logic_vector(8 downto 0) := (others => '0');

--//-- TEST POINTS, LEDs
signal LED_CNTR3: integer range 0 to 2 := 0;
signal LED_CNTR4: integer range 0 to 2 := 0;

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_1_CLK: std_logic := '0';
signal S1_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_2_CLK: std_logic := '0';
signal S1_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_3_CLK: std_logic := '0';
signal S1_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S1_4_CLK: std_logic := '0';
signal S2_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_1_CLK: std_logic := '0';
signal S2_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_2_CLK: std_logic := '0';
signal S2_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_3_CLK: std_logic := '0';
signal S2_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S2_4_CLK: std_logic := '0';
signal S3_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_1_CLK: std_logic := '0';
signal S3_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_2_CLK: std_logic := '0';
signal S3_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_3_CLK: std_logic := '0';
signal S3_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S3_4_CLK: std_logic := '0';
signal S4_1: std_logic_vector(7 downto 0) := (others => '0');	
signal S4_1_CLK: std_logic := '0';
signal S4_2: std_logic_vector(7 downto 0) := (others => '0');	
signal S4_2_CLK: std_logic := '0';
signal S4_3: std_logic_vector(7 downto 0) := (others => '0');	
signal S4_3_CLK: std_logic := '0';
signal S4_4: std_logic_vector(7 downto 0) := (others => '0');	
signal S4_4_CLK: std_logic := '0';

signal T16b_1: std_logic_vector(15 downto 0) := (others => '0');	
signal T16b_1_VALID: std_logic := '0';
signal T16b_2: std_logic_vector(15 downto 0) := (others => '0');	
signal T16b_2_VALID: std_logic := '0';

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal SREG250_READ_TOGGLE: std_logic := '0';
signal COMSCOPE2_TP: std_logic_vector(16 downto 1) := (others => '0');

-- DNA Port ------------------------------------
signal DNA_ID_DATA: std_logic_vector(56 downto 0) := (others => '0');
signal DNA_ID_SAMPLE_CLK: std_logic := '0';

--// ARM co-processor interface
signal UC_WEBG: std_logic := '0';  -- WE# through global buffer
signal UC_REBG: std_logic := '0';  -- RE# through global buffer
signal UC_ADDRESS: unsigned(7 downto 0) := (others => '0');
signal UC_AD_local: std_logic_vector(7 downto 0) := (others => '0');
signal UC_RDN_D: std_logic := '0';
signal UC_RDN_D2: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE1_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE1_D2: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE2_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE2_D2: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE4_D: std_logic := '0';
signal CONFIG_CHANGE_TOGGLE4_D2: std_logic := '0';
signal CONFIG_CHANGE_PULSE_CLK_P: std_logic := '0';
signal CONFIG_CHANGE_PULSE_CLK_P_D: std_logic := '0';
signal CONFIG_CHANGE_PULSE_CLK_TXg: std_logic := '0';
signal CONFIG_CHANGE_PULSE_CLK_TXg_D: std_logic := '0';
signal CONFIG_CHANGE_PULSE_CLK_RXg: std_logic := '0';
signal MOD_TX_LEVEL_CHANGE_TOGGLE: std_logic := '0';
signal MOD_TX_LEVEL_CHANGE_TOGGLE_D: std_logic := '0';
signal MOD_TX_LEVEL_CHANGE_TOGGLE_D2: std_logic := '0';
signal MOD_TX_LEVEL_CHANGE_PULSE: std_logic := '0';
signal RF_AGC_LEVEL_CHANGE_TOGGLE: std_logic := '0';
signal RF_AGC_LEVEL_CHANGE_TOGGLE_D: std_logic := '0';
signal RF_AGC_LEVEL_CHANGE_TOGGLE_D2: std_logic := '0';
signal IF_AGC_LEVEL_CHANGE_TOGGLE: std_logic := '0';
signal IF_AGC_LEVEL_CHANGE_TOGGLE_D: std_logic := '0';
signal IF_AGC_LEVEL_CHANGE_TOGGLE_D2: std_logic := '0';

--// Control Registers
-- TODO: define the number of control registers
constant NCREGS: integer := 10;	-- number of control registers.
constant NCREG_IMIN: integer := 39;	-- starting at this index
constant NCREG_IMAX: integer := (NCREG_IMIN+NCREGS-1);
type SLV8xNCREGStype is array (integer range NCREG_IMIN to NCREG_IMAX) of std_logic_vector(7 downto 0);
signal CREG: SLV8xNCREGStype;
signal REG200: std_logic_vector(7 downto 0) := x"00";   -- RF frequency synthesizers controls
signal REG201: std_logic_vector(7 downto 0) := x"00";   
signal REG202: std_logic_vector(7 downto 0) := x"00";   
signal REG203: std_logic_vector(7 downto 0) := x"00";   
signal REG204: std_logic_vector(7 downto 0) := x"00";   
signal REG255: std_logic_vector(7 downto 0) := (others => '0');
signal UC_CONFIGURATION_DONE: std_logic := '0';

--// Status Registers
constant NSREGS: integer := 10;	-- TODO: set the number of status registers for your application.
constant NSREG_IMIN: integer := 16;	-- starting at this index
constant NSREG_IMAX: integer := (NSREG_IMIN+NSREGS-1);
type SLV8xNSREGStype is array (integer range NSREG_IMIN to NSREG_IMAX) of std_logic_vector(7 downto 0);
signal SREG: SLV8xNSREGStype := (others => (others => '0'));
signal SREG16_READ_TOGGLE: std_logic := '0';
signal SREG16_READ_TOGGLE1_D: std_logic := '0';
signal SREG16_READ_TOGGLE1_D2: std_logic := '0';
signal SREG16_READ_TOGGLE2_D: std_logic := '0';
signal SREG16_READ_TOGGLE2_D2: std_logic := '0';
signal SREG16_READ_TOGGLE3_D: std_logic := '0';
signal SREG16_READ_TOGGLE3_D2: std_logic := '0';
signal SREG254_READ_TOGGLE: std_logic := '0';			
signal SREG254_READ_TOGGLE_D: std_logic := '0';			
signal SREG254_READ_TOGGLE_D2: std_logic := '0';	
signal LATCH_MONITORING_PULSE: std_logic := '0';	 	
signal LATCH_MONITORING_PULSE_CLK_RX: std_logic := '0';	 	
signal LATCH_MONITORING_PULSE_CLK_TX: std_logic := '0';	 	

--// ComScope control registers
--signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal SREG250: std_logic_vector(7 downto 0):= "00000000";
signal SREG251: std_logic_vector(7 downto 0):= "00000000";


constant CLK_FREQUENCY: integer := 120;	-- fclk_p
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin
------------------------------------------------------------------------------------------------------------------------
--// CLOCKS, RESET 
------------------------------------------------------------------------------------------------------------------------
REF_CLK_GEN_001: if(SIMULATION = '1') generate
    -- 10 MHz
    process
    begin
        CLK10g <= '0';
        wait for 50 ns;
        CLK10g <= '1';
        wait for 50 ns;
    end process;
    CLKGEN7C_LOCKED <= '1';
    CLKGEN7C_INPUT_CLK_STOPPED <= '0';
    
    -- 19.2 MHz
    process
    begin
        CLKREF_TCXOg <= '0';
        wait for 26.041666 ns;
        CLKREF_TCXOg <= '1';
        wait for 26.041666 ns;
    end process;
    
    -- loopback ADC clock
    CLK_RXg <= CLK_ADCg;
end generate;
REF_CLK_GEN_002: if(SIMULATION = '0') and (OPTION = x"41") generate
    -- The main clock reference is the 19.2MHz TCXO clock (also used by the ARM processor).
    BUFG_001: BUFG port map (I => CLKREF_TCXO, O => CLKREF_TCXOg);
	 CLK_RXg <= CLK_ADCg;

     CLKGEN7C: CLKGEN7_MMCM_ADJ 
    GENERIC MAP(
		CLKFBOUT_MULT_F => 50.000,
		CLKOUT0_DIVIDE_F => 96.000,
		CLKIN1_PERIOD => 52.083
    ) 
    PORT MAP(
		CLK_IN1 => CLKREF_TCXOg,    -- 19.2 MHz TCXO clock 
		CLK_OUT1 => CLK10g,    -- 10 MHz global
		PHASE_SHIFT_VAL => x"00",
		PHASE_SHIFT_TRIGGER_TOGGLE => '0',
		INPUT_CLK_STOPPED => CLKGEN7C_INPUT_CLK_STOPPED,
		LOCKED => CLKGEN7C_LOCKED
    );
end generate;
REF_CLK_GEN_003: if(SIMULATION = '0') and (OPTION = x"42") generate
    CLK10g_GEN: BUFG port map (I => CLKREF_EXT, O => CLK10g);
    CLKREF_TCXOg_GEN: BUFG port map(I => CLKREF_TCXO, O=> CLKREF_TCXOg);
	 CLK_RXg <= CLK_ADCg;	-- better at 160 MHz than using the ADC_DCO_IN_LVDS from the ADC  8/25/15

end generate;

-- Option -A. Internal TCXO frequency reference.
-- Generate 120 MHz CLK_P, 160 MHz CLK_ADCg, 10 MHz CLK10g
OPTION_A_001: if(OPTION = x"41") generate

   -- generate CLK_ADC 160 MHz sampling clock 
	-- limited by place and route
	-- IF bandpass filter is centered at 200 MHz = (about)160*1.25 for IF undersampling
    CLKGEN7A: CLKGEN7_MMCM_ADJ 
    GENERIC MAP(
		CLKFBOUT_MULT_F => 50.000,
		CLKOUT0_DIVIDE_F => 6.000,
		CLKIN1_PERIOD => 52.083
    ) 
    PORT MAP(
		CLK_IN1 => CLKREF_TCXOg,    -- 19.2 MHz TCXO clock 
		CLK_OUT1 => CLK_ADCg,    -- 160 MHz global
		PHASE_SHIFT_VAL => x"00",
		PHASE_SHIFT_TRIGGER_TOGGLE => '0',
		INPUT_CLK_STOPPED => CLKGEN7A_INPUT_CLK_STOPPED,
		LOCKED => CLKGEN7A_LOCKED
    );
    
    -- generate CLK_P processing clock 120 MHz, almost enough to get the maximum GbE LAN throughput.
    -- Note: 125 MHz is the user processing clock for a full 1Gbps LAN stream.
    -- See ug472.pdf for details.
    CLKGEN7B: CLKGEN7_MMCM_ADJ 
    GENERIC MAP(
		CLKFBOUT_MULT_F => 50.000,
		CLKOUT0_DIVIDE_F => 8.000,
		CLKIN1_PERIOD => 52.083
    ) 
    PORT MAP(
		CLK_IN1 => CLKREF_TCXOg,	-- 19.2 MHz TCXO clock 
		CLK_OUT1 => CLK_P,	-- 120 MHz global
		PHASE_SHIFT_VAL => x"00",
		PHASE_SHIFT_TRIGGER_TOGGLE => '0',
		INPUT_CLK_STOPPED => CLKGEN7B_INPUT_CLK_STOPPED,
		LOCKED => CLKGEN7B_LOCKED
    );

    -- Generate miscellaneous global clocks and resets from the always present CLKREF_TCXOg
    RESET_STATE_MACHINE_001: process(CLKREF_TCXOg)
    begin
        if rising_edge(CLKREF_TCXOg) then
            -- PLL need time to lock. The reset of the application is still in reset
            if(RESET_COUNTER(17) = '0') and (SIMULATION = '0') then
                -- wait a bit before asserting the PLL lock
                RESET_COUNTER <= RESET_COUNTER + 1;
            elsif(RESET_COUNTER(5) = '0') and (SIMULATION = '1') then  
                -- make it short during simulations
                -- wait a bit before asserting the PLL lock
                RESET_COUNTER <= RESET_COUNTER + 1;
            elsif(CLKGEN7A_LOCKED = '1') and (CLKGEN7B_LOCKED = '1') and (CLKGEN7C_LOCKED = '1') then
                -- assert the PLL to lock
                RESET_STATE <= 1;
            elsif(RESET_COUNTER(5) = '1') and (SIMULATION = '1') then  
                -- simulation only. Don't waste simulation time waiting for PLL lock
                RESET_STATE <= 1;
            end if;
            
            if(RESET_STATE = 1) then
                ASYNC_RESET <= '0';
            else
                ASYNC_RESET <= '1';
            end if;
        end if;
    end process;
    
end generate;

-- Option -B. External 10MHz frequency reference.
-- Generate 120 MHz CLK_P, 160 MHz CLK_ADCg, 10 MHz CLK10g
OPTION_B_001: if(OPTION = x"42") generate

    -- generate CLK_ADC 160 MHz high-speed IO clock 
    -- limited by place and route
    -- IF bandpass filter is centered at 200 MHz = 160*1.25 for IF undersampling
    CLKGEN7A: CLKGEN7_MMCM_ADJ 
    GENERIC MAP(
		CLKFBOUT_MULT_F => 64.000,
		CLKOUT0_DIVIDE_F => 4.000,
		CLKIN1_PERIOD => 100.000
    ) 
    PORT MAP(
		CLK_IN1 => CLK10g,    -- 10 MHz frequency reference
		CLK_OUT1 => CLK_ADCg,    -- 160 MHz global
		PHASE_SHIFT_VAL => x"00",
		PHASE_SHIFT_TRIGGER_TOGGLE => '0',
		INPUT_CLK_STOPPED => CLKGEN7A_INPUT_CLK_STOPPED,
		LOCKED => CLKGEN7A_LOCKED
    );
   
    -- generate CLK_P processing clock 120 MHz, almost enough to get the maximum GbE LAN throughput.
    -- Note: 125 MHz is the user processing clock for a full 1Gbps LAN stream.
    -- See ug472.pdf for details.
    CLKGEN7B: CLKGEN7_MMCM_ADJ 
    GENERIC MAP(
		CLKFBOUT_MULT_F => 60.000,
		CLKOUT0_DIVIDE_F => 5.000,
		CLKIN1_PERIOD => 100.00
    ) 
    PORT MAP(
		CLK_IN1 => CLK10g,    -- 10 MHz frequency reference
		CLK_OUT1 => CLK_P,    -- 120 MHz global
		PHASE_SHIFT_VAL => x"00",
		PHASE_SHIFT_TRIGGER_TOGGLE => '0',
		INPUT_CLK_STOPPED => CLKGEN7B_INPUT_CLK_STOPPED,
		LOCKED => CLKGEN7B_LOCKED
    );

    -- Generate miscellaneous global clocks and resets from the always present 10MHz
    RESET_STATE_MACHINE_001: process(CLK10g)
    begin
        if rising_edge(CLK10g) then
            -- PLL need time to lock. The reset of the application is still in reset
            if(RESET_COUNTER(17) = '0') and (SIMULATION = '0') then
                -- wait a bit before asserting the PLL lock
                RESET_COUNTER <= RESET_COUNTER + 1;
            elsif(RESET_COUNTER(5) = '0') and (SIMULATION = '1') then  
                -- make it short during simulations
                -- wait a bit before asserting the PLL lock
                RESET_COUNTER <= RESET_COUNTER + 1;
            elsif(CLKGEN7A_LOCKED = '1') and (CLKGEN7B_LOCKED = '1') then
                -- assert the PLL to lock
                RESET_STATE <= 1;
            elsif(RESET_COUNTER(5) = '1') and (SIMULATION = '1') then  
                -- simulation only. Don't waste simulation time waiting for PLL lock
                RESET_STATE <= 1;
            end if;
            
            if(RESET_STATE = 1) then
                ASYNC_RESET <= '0';
            else
                ASYNC_RESET <= '1';
            end if;
        end if;
    end process;
    
end generate;


CLK10_OUT_GEN_001 : ODDR
generic map(
	DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => CLKREF_OUT,   -- 1-bit DDR output
	C => CLK10g,    -- 1-bit clock input
	CE => CREG(39)(0),  -- 1-bit clock enable input
	D1 => '1',  -- 1-bit data input (positive edge)
	D2 => '0',  -- 1-bit data input (negative edge)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

CLK10_OUT_GEN_002 : ODDR
generic map(
	DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => CLK_LNB,   -- 1-bit DDR output
	C => CLK10g,    -- 1-bit clock input
	CE => CREG(39)(1),  -- 1-bit clock enable input
	D1 => '1',  -- 1-bit data input (positive edge)
	D2 => '0',  -- 1-bit data input (negative edge)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

CLK10_OUT_GEN_003 : ODDR
generic map(
	DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => CLK_TX_OUT,   -- 1-bit DDR output
	C => CLK10g,    -- 1-bit clock input
	CE => CREG(39)(2),  -- 1-bit clock enable input
	D1 => '1',  -- 1-bit data input (positive edge)
	D2 => '0',  -- 1-bit data input (negative edge)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);



-- send ADC clock out
-- ADC clock output
-- It is recommended to use a Clock Forwarding technique to create a reliable and repeatable 
-- low skew solution, i.e. use ODDR to regenerate the output clock.
-- FAKE differential (LVDS output not supported in Bank 15 (HR) with 1.8V VCCO)
ADC_CLKOUT_GEN_001P : ODDR
generic map(
	DDR_CLK_EDGE => "SAME_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "ASYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => CLK_ADC_OUT, -- 1-bit output data
	C => CLK_ADCg, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	-- inversion for better timing. verified for 160 MSamples/s
	D1 => '0',   -- 1-bit data input (associated with C0)
	D2 => '1',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

-- ADC sampling clock (To ADC)
OBUFDS_inst : OBUFDS
generic map (
	IOSTANDARD => "LVDS_25", -- Specify the output I/O standard
	SLEW => "SLOW")          -- Specify the output slew rate
port map (
	O => ADC_CLK_OUT_LVDS(1),     -- Diff_p output (connect directly to top-level port)
	OB => ADC_CLK_OUT_LVDS(0),   -- Diff_n output (connect directly to top-level port)
	I => CLK_ADC_OUT      -- Buffer input 
);


-- ADC sampling clock (back from ADC)
--ADC_DCO_IN_001 : IBUFDS
--generic map (
--	DIFF_TERM => TRUE, -- Differential Termination. Fake 2.5V VCCO (TBC)
--	IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
--	IOSTANDARD => "LVDS_25")	-- must be LVDS_25 because bank15 is HR 
--port map (
--	I => ADC_DCO_IN_LVDS(1),     -- Diff_p buffer input (connect directly to top-level port)
--	IB => ADC_DCO_IN_LVDS(0),   -- Diff_n buffer input (connect directly to top-level port)
--	O => ADC_CLK_IN     -- ADC sampling clock 
--);
-- better to use CLK_ADCg at 160 MHz than using the ADC_DCO_IN_LVDS from the ADC  8/25/15


	-- external input clock
BUFG_005: BUFG  port map (I=> UC_WEB_IN, O=> UC_WEBG);	
	-- ARM WE#
BUFG_006: BUFG  port map (I=> UC_REB_IN, O=> UC_REBG);	
	-- ARM RE#

Inst_TIMER_4US: TIMER_4US 
GENERIC MAP(
	CLK_FREQUENCY => CLK_FREQUENCY
)
PORT MAP(
	CLK => CLK_P,
	SYNC_RESET => '0',
	TICK_4US => open,
	TICK_100MS => TICK_100MS_rt
);

	
-- modulo-10 counter. 1s tick
TICK1S_002: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(SYNC_RESET_CLK_P = '1') then
			TICK_100MS_CNTR <= "0000";
			TICK_1S <= '0';
		elsif(TICK_100MS_rt = '1') then
			if(TICK_100MS_CNTR = 0) then
				TICK_100MS_CNTR <= "1001";
				TICK_1S <= '1';
			else
				TICK_100MS_CNTR <= TICK_100MS_CNTR - 1;
				TICK_1S <= '0';
			end if;
		else
			TICK_1S <= '0';
		end if;
	end if;
end process;


--//-- TX PLL ------------------------------------------------
-- generate the modulator processing clock CLK_TXg from a 120 MHz (locked to 19.2 MHz TCXO)
-- Note: this code only works on series-7 Xilinx FPGA, using a runtime-programmable MMCM clock manager (or PLL with some minor modifications).
-- For older platforms, use HIGH_SPEED_NCO3 component.

-- TODO: set the DAC control frequency for your application. The frequency can be changed dynamically at runtime
-- by changing the D,M and O variables below, then sending a CLK_TX_GEN_PROGRAM pulse.
-- Example for 160 MHz = 120MHz * 8/6
CLK_TX_GEN_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
        CLK_TX_GEN_D <= "0000001";  -- range 1 - 106
        CLK_TX_GEN_M <= "0001000000";  -- range 2.0 - 64.0 in increment steps of 0.125 fixed point format 7.3 
        CLK_TX_GEN_O <= "00000110000";  -- range 2.0 to 128.0 in increment steps of 0.125 fixed point format 8.3
    end if;
end process;

CLK_TX_GEN_PROGRAM <= SYNC_RESET_CLK_P or CONFIG_CHANGE_PULSE_CLK_P;
CLK_TX_GEN_SYNC_RESET <= SYNC_RESET_CLK_P when (SIMULATION = '0') else '0';  -- MANDATORY (for implementation), may affect simulation (replace with '0') unless simulation creates a config_change pulse.
CLK_TX_GEN_002: CLKGEN7_MMCM_DYNAMIC 
GENERIC MAP(
  CLKIN1_PERIOD => 8.333        -- 120 MHz frequency reference
)    
PORT MAP(
	CLK_IN1 => CLK_P,	-- 120 MHz frequency reference
	SYNC_RESET => CLK_TX_GEN_SYNC_RESET,    
	D => CLK_TX_GEN_D,  -- range 1 - 106
	M => CLK_TX_GEN_M,   -- range 2.0 - 64.0 in increment steps of 0.125 fixed point format 7.3 
	O => CLK_TX_GEN_O,   -- range 2.0 to 128.0 in increment steps of 0.125 fixed point format 8.3
	PROGRAM => CLK_TX_GEN_PROGRAM, 
	CLK_OUT1 => CLK_TXg,	-- transmit clock (2^n * modulation symbol rate), GLOBAL
	INPUT_CLK_STOPPED => CLK_TX_GEN_INPUT_CLK_STOPPED,
	LOCKED => CLK_TX_GEN_LOCKED
);


-- generate reset pulse synchronous with processing clock CLK_P
SYNC_RESET_CLK_TXg_GEN: process(CLK_TXg)
begin
	if rising_edge(CLK_TXg) then
		ASYNC_RESET3_D <= ASYNC_RESET;
		ASYNC_RESET3_D2 <= ASYNC_RESET3_D;
		if(ASYNC_RESET3_D2 = '1') and (ASYNC_RESET3_D = '0') then
			SYNC_RESET_CLK_TXgA <= '1';
		else
			SYNC_RESET_CLK_TXgA <= '0';
		end if;
	end if;
end process;
SYNC_RESET_CLK_TXg <= SYNC_RESET_CLK_TXgA or (not CLK_TX_GEN_LOCKED);	-- keep tx circuit in reset until CLK_TXg is stable


-- generate a reset synchronous with CLK_P at the end of the ASYNC_RESET
SYNC_RESET_CLK_P_GEN: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		ASYNC_RESET_D <= ASYNC_RESET;
		ASYNC_RESET_D2 <= ASYNC_RESET_D;
		
		if(ASYNC_RESET_D = '0') and (ASYNC_RESET_D2 = '1')  then
			SYNC_RESET_CLK_P <= '1';
		else
			SYNC_RESET_CLK_P <= '0';
		end if;
		
		-- '0' from power-up/async_reset until FPGA is ready
		if(SYNC_RESET_CLK_P = '1') then
			READY_FLAG_CLK_P <= '1';
		end if;
	end if;
end process;

-- generate a reset synchronous with CLK_RXg at the end of the ASYNC_RESET
SYNC_RESET_CLK_RXg_GEN: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		ASYNC_RESET2_D <= ASYNC_RESET;
		ASYNC_RESET2_D2 <= ASYNC_RESET2_D;
		
		if(ASYNC_RESET2_D = '0') and (ASYNC_RESET2_D2 = '1')  then
			SYNC_RESET_CLK_RXg <= '1';
		else
			SYNC_RESET_CLK_RXg <= '0';
		end if;
	end if;
end process;

------------------------------------------------------------------------------------------------------------------------
--// RF FREQUENCY SYNTHESIZERS ---------------------
------------------------------------------------------------------------------------------------------------------------
-- Programmed by the ARM micro through control register REG200
-- reclock with CLK_P for easier P&R
UC_WRITE_001e: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		RX_SYNTH_CTRL(0) <= REG200(0);	-- CLK
		RX_SYNTH_CTRL(1) <= REG200(1);	-- DATA
		RX_SYNTH_CTRL(2) <= REG200(2);	-- LE
		RX_SYNTH_CTRL(4) <= REG200(3);	-- PDBRF
		TX_SYNTH_CTRL(0) <= REG200(4);	-- CLK
		TX_SYNTH_CTRL(1) <= REG200(5);	-- DATA
		TX_SYNTH_CTRL(2) <= REG200(6);	-- LE
		TX_SYNTH_CTRL(4) <= REG200(7) and CREG(39)(3);	-- PDBRF
	end if;
end process;

	-- monitoring
MONITORING_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		-- RF synthesizers lock status
		SREG(19)(0) <= RX_SYNTH_CTRL(3);
		SREG(19)(1) <= TX_SYNTH_CTRL(3);
	end if;
end process;

------------------------------------------------------------------------------------------------------------------------
--// RECEIVER SIDE
------------------------------------------------------------------------------------------------------------------------

--// LOOP BACK -----------------------------------------------

--  Cross clock domains from transmitter (CLK_TXg) to receiver (CLK_RXg) Keep only 14 MSBs.
CROSS_CLK_DOMAINS_001: CROSS_CLK_DOMAINS 
generic map(
	NBITS => 14
)
PORT MAP(
	ASYNC_RESET => ASYNC_RESET,
	CLK_A => CLK_TXg,  -- modulator global clock
	CLK_B => CLK_RXg,	 -- demodulator global clock
	DATA_IN => std_logic_vector(TEST_RAMP1),  
	DATA_OUT => LOOPBACK_DATA1_I
);

CROSS_CLK_DOMAINS_002: CROSS_CLK_DOMAINS 
generic map(
	NBITS => 14
)
PORT MAP(
	ASYNC_RESET => ASYNC_RESET,
	CLK_A => CLK_TXg,  -- modulator global clock
	CLK_B => CLK_RXg,	 -- demodulator global clock
	DATA_IN => std_logic_vector(TEST_RAMP2),  
	DATA_OUT => LOOPBACK_DATA1_Q
);

--// AUX DACs (RF/IF AGC) ---------------------------------------------------------
-- RF AGC gain control (ARM -> FPGA -> Aux DAC1)
DAC_AD5621_001a: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		RF_AGC_LEVEL_CHANGE_TOGGLE_D <= RF_AGC_LEVEL_CHANGE_TOGGLE;
		RF_AGC_LEVEL_CHANGE_TOGGLE_D2 <= RF_AGC_LEVEL_CHANGE_TOGGLE_D;
		
		if(RF_AGC_LEVEL_CHANGE_TOGGLE_D /= RF_AGC_LEVEL_CHANGE_TOGGLE_D2) then
			-- latch multi-byte value upon reading the MSB
			RF_AGC_DAC <= REG202(3 downto 0) & REG201;
			RF_AGC_DAC_SAMPLE_CLK <= '1';
		else
			RF_AGC_DAC_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;

DAC_AD5621_001b: DAC_AD5621 
generic map(
	CLK_FREQUENCY => 160	-- 160 MHz CLK_RXg frequency 
)
PORT MAP(
	CLK => CLK_RXg,							-- synchronous clock
	DATA_IN => RF_AGC_DAC,		-- user 12-bit input data
	DATA_IN_CLK => RF_AGC_DAC_SAMPLE_CLK,		-- user enable
	OP_MODE(0) => '0',			-- "00" for normal DAC operation, "11" for high impedance output
	OP_MODE(1) => '0',
	DONE => open,				-- tell user when the DAC is ready for the next input sample
	--// DAC interface
	SYNC_B => AUX_DAC1_S(3),					-- DAC serial connections. Do not modify.
	SCLK => AUX_DAC1_S(2),						-- DAC serial connections. Do not modify.
	SDIN => AUX_DAC1_S(1)						-- DAC serial connections. Do not modify.
	);

-- IF AGC gain control (ARM -> FPGA -> Aux DAC1)
DAC_AD5621_002a: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		IF_AGC_LEVEL_CHANGE_TOGGLE_D <= IF_AGC_LEVEL_CHANGE_TOGGLE;
		IF_AGC_LEVEL_CHANGE_TOGGLE_D2 <= IF_AGC_LEVEL_CHANGE_TOGGLE_D;
		
		if(IF_AGC_LEVEL_CHANGE_TOGGLE_D /= IF_AGC_LEVEL_CHANGE_TOGGLE_D2) then
			-- latch multi-byte value upon reading the MSB
			IF_AGC_DAC <= REG204(3 downto 0) & REG203;
			IF_AGC_DAC_SAMPLE_CLK <= '1';
		else
			IF_AGC_DAC_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;

DAC_AD5621_002b: DAC_AD5621 
generic map(
	CLK_FREQUENCY => 160	-- 160 MHz CLK_RXg frequency 
)
PORT MAP(
	CLK => CLK_RXg,							-- synchronous clock
	DATA_IN => IF_AGC_DAC,		-- user 12-bit input data
	DATA_IN_CLK => IF_AGC_DAC_SAMPLE_CLK,		-- user enable
	OP_MODE(0) => '0',			-- "00" for normal DAC operation, "11" for high impedance output
	OP_MODE(1) => '0',
	DONE => open,				-- tell user when the DAC is ready for the next input sample
	--// DAC interface
	SYNC_B => AUX_DAC2_S(3),					-- DAC serial connections. Do not modify.
	SCLK => AUX_DAC2_S(2),						-- DAC serial connections. Do not modify.
	SDIN => AUX_DAC2_S(1)						-- DAC serial connections. Do not modify.
	);

--// ADCs ---------------------------------------------------------
-- Immediately reclock samples in IOBs

-- ADC input samples are LVDS/DDR
ADC_D_LVDS_001: for I in 0 to 6 generate
	IBUFDS_X : IBUFDS
	generic map (
		DIFF_TERM => TRUE, -- Differential Termination. Fake 2.5V VCCO (TBC)
      IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD => "LVDS_25")	-- must be LVDS_25 because bank15 is HR 
	port map (
		I => ADC_D_IN_LVDS(2*I + 1),     -- Diff_p buffer input (connect directly to top-level port)
		IB => ADC_D_IN_LVDS(2*I),   -- Diff_n buffer input (connect directly to top-level port)
		O => ADC_D_IN_DDR(I)     -- DDR ADC samples 
	);
end generate;

-- demux DDR input
ADC_D_DDR_001: for I in 0 to 6 generate
   IDDR_X : IDDR 
   generic map (
      DDR_CLK_EDGE => "SAME_EDGE_PIPELINED", -- "OPPOSITE_EDGE", "SAME_EDGE" 
                                       -- or "SAME_EDGE_PIPELINED" 
      INIT_Q1 => '0', -- Initial value of Q1: '0' or '1'
      INIT_Q2 => '0', -- Initial value of Q2: '0' or '1'
      SRTYPE => "SYNC") -- Set/Reset type: "SYNC" or "ASYNC" 
   port map (
      Q1 => ADC_D_IN(2*I), -- 1-bit output for positive edge of clock 
      Q2 => ADC_D_IN((2*I)+1), -- 1-bit output for negative edge of clock
      C => CLK_RXg,   -- 1-bit clock input
      CE => '1', -- 1-bit clock enable input
      D => ADC_D_IN_DDR(I),   -- 1-bit DDR data input
      R => '0',   -- 1-bit reset
      S => '0'    -- 1-bit set
      );
end generate;
 
-- Extend precision to 14 bits.
ADC_DATA_IN_001: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		ADC_DATA_IN(13 downto 2) <= ADC_D_IN(13 downto 2);
		ADC_DATA_IN(1 downto 0) <= "00";
		-- The lower 2 bits are OR+/OR- signals when using the 12-bit AD9634.
		-- The lower 2 bits are LSbs when using the 14-bit AD9642.
	end if;
end process;
 
-- hardware check: configure the DCO clock delay, then read the AD9634 chip id at reg0x1. Expecting x87
-- see AD9634 specifications for registers definitions.
-- Note: tried to control regs x16 (invert DCO clock) and x17 (DCO output delay) without success.
ADC_SPI_001:process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		if(SYNC_RESET_CLK_RXg = '1') then
			ADC_SPI_STATE <= 0;
			ADC_SPI_WRITE_REQ <= '0';
			ADC_SPI_READ_REQ <= '0';
--		elsif(ADC_SPI_STATE = 0) and (ADC_SPI_BUSY = '0') then
--			-- write to register x16 invert DCO clock?
--			ADC_SPI_WRITE_REQ <= '1';
--			ADC_SPI_STATE <= ADC_SPI_STATE + 1;
--			ADC_SPI_ADDR <= "0000000010110";	-- x16
--			ADC_SPI_DATA_OUT <= x"00";	-- invert DCO clock?
--		elsif(ADC_SPI_STATE = 1) and (ADC_SPI_BUSY = '0') then
--			-- write to register x17 DCO output delay
--			ADC_SPI_WRITE_REQ <= '1';
--			ADC_SPI_STATE <= ADC_SPI_STATE + 1;
--			ADC_SPI_ADDR <= "0000000010111";	-- x17
--			ADC_SPI_DATA_OUT <= x"00";	--  DCO output delay
--		elsif(ADC_SPI_STATE = 2) and (ADC_SPI_BUSY = '0') then
--			-- transfer command
--			ADC_SPI_WRITE_REQ <= '1';
--			ADC_SPI_STATE <= ADC_SPI_STATE + 1;
--			ADC_SPI_ADDR <= "0000011111111";	-- xFF
--			ADC_SPI_DATA_OUT <= x"01";
		elsif(ADC_SPI_STATE = 0) and (ADC_SPI_BUSY = '0') then
			-- read from register x01 chip id. Expecting x87
			ADC_SPI_READ_REQ <= '1';
			ADC_SPI_STATE <= ADC_SPI_STATE + 1;
			ADC_SPI_ADDR <= "0000000000001";	-- x01
		else
			ADC_SPI_WRITE_REQ <= '0';
			ADC_SPI_READ_REQ <= '0';
		end if;
	end if;
end process;

ADC_SPI_002: SPI 
generic map(
	CONTROL_WIDTH => 16,	-- 16-bit control field
	NDIV => 32)	-- 25 MHz max SPI clk
PORT MAP (
		 CLK => CLK_RXg,
		 ASYNC_RESET => '0',
		 ADDR => ADC_SPI_ADDR,	
		 DATA_OUT => ADC_SPI_DATA_OUT,
		 WRITE_REQ => ADC_SPI_WRITE_REQ,
		 READ_REQ => ADC_SPI_READ_REQ,
		 DATA_IN => ADC_SPI_DATA_IN,
		 DATA_IN_CLK => ADC_SPI_DATA_IN_CLK,
		 BUSY => ADC_SPI_BUSY,
		 CSB => ADC_SPI(3),
		 SCLK => ADC_SPI(1),
		 SDIO => ADC_SPI(2)
	  );

SAVE_SREG26: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		if (ADC_SPI_DATA_IN_CLK = '1') then
			SREG(18) <= ADC_SPI_DATA_IN;
		end if;
	end if;
end process;

--// RECEIVER INPUT PROCESSING -------------------------------------
-- Place your rx digital signal processing here. 
-- Input is ADC_DATA_IN: 12-bit real input from ADC. Format: 2's complement signed. IF undersampling.
-- Sampling clock: CLK_RXg
INPUT_DSP_001: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		-- IF undersampling
		-- 12-bit real input from ADC. Format: 2's complement signed. 
		RX_DATA1_I <= ADC_DATA_IN;  
		RX_DATA1_Q <= (others => '0');	-- IF undersampling. Force Q to zero
	end if;
end process;

-- ....

------------------------------------------------------------------------------------------------------------------------
--// DUAL 175MS/S DAC ------------------
------------------------------------------------------------------------------------------------------------------------
-- TODO: replace test DAC signals with actual application I/Q signals
-- test RAMPS to DACs
RAMP_001: process(CLK_TXg)
begin
	if rising_edge(CLK_TXg) then
		TEST_RAMP1 <= TEST_RAMP1 + 1;
		TEST_RAMP2 <= TEST_RAMP2 - 1;
	end if;
end process;

-- convert from 2's complement to unsigned (offset binary)
DACx_DOUT: process(CLK_TXg)
begin
	if rising_edge(CLK_TXg) then
        DAC1_DOUT(13) <= not TEST_RAMP1(13);
        DAC1_DOUT(12 downto 0) <= std_logic_vector(TEST_RAMP1(12 downto 0));
        DAC2_DOUT(13) <= not TEST_RAMP2(13);
        DAC2_DOUT(12 downto 0) <= std_logic_vector(TEST_RAMP2(12 downto 0));
	end if;
end process;

	-- DAC clock output
	-- control the clock phase to minimize glitches
	-- regenerate CLK_TXg
	PLLE2_ADV_DAC : PLLE2_ADV
	generic map
	 (BANDWIDTH            => "LOW",

	  
	  COMPENSATION         => "ZHOLD",
	  DIVCLK_DIVIDE        => 1,
	  CLKFBOUT_MULT        => 10,
	  CLKFBOUT_PHASE       => 0.000,
	  CLKOUT0_DIVIDE       => 10,
	  CLKOUT0_PHASE        => 45.0,
	  CLKOUT0_DUTY_CYCLE   => 0.500,
	  CLKIN1_PERIOD        => 6.250)	-- 160 MHz maximum clock frequency
	port map
	  -- Output clocks
	 (
	  CLKFBOUT            => CLK_TXg_FB,
	  CLKOUT0             => CLK_TXg2,
	  CLKOUT1             => open,
	  CLKOUT2             => open,
	  CLKOUT3             => open,
	  CLKOUT4             => open,
	  CLKOUT5             => open,
	  -- Input clock control
	  CLKFBIN             => CLK_TXg_FB,
	  CLKIN1              => CLK_TXg,
	  CLKIN2              => '0',
	  -- Tied to always select the primary input clock
	  CLKINSEL            => '1',
	  -- Ports for dynamic reconfiguration
	  DADDR               => (others => '0'),
	  DCLK                => '0',
	  DEN                 => '0',
	  DI                  => (others => '0'),
	  DO                  => open,
	  DRDY                => open,
	  DWE                 => '0',
	  -- Other control and status signals
	  LOCKED              => CLK_TXg2_LOCKED,
	  PWRDWN              => '0',
	  RST                 => SYNC_RESET_CLK_TXg
	  );


DAC_CLKOUT_GEN_001 : ODDR
generic map(
	DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => DAC_CLKOUT_P,   -- 1-bit DDR output
	C => CLK_TXg2,    -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D1 => '0',  -- 1-bit data input (positive edge)
	D2 => '1',  -- 1-bit data input (negative edge)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

DAC_CLKOUT_GEN_002 : ODDR
generic map(
	DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
	INIT => '0',   -- Initial value for Q port ('1' or '0')
	SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
port map (
	Q => DAC_CLKOUT_N,   -- 1-bit DDR output
	C => CLK_TXg2,    -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D1 => '1',  -- 1-bit data input (positive edge)
	D2 => '0',  -- 1-bit data input (negative edge)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

DAC_SPI(3) <= '1';	-- pin mode
DAC_SPI(1) <= '1';	-- differential clock 
DAC_SPI(2) <= '0';	-- straight binary format
DAC1_SPI_CSB <= '0';	-- no sleep
DAC2_SPI_CSB <= '0';	-- no sleep

------------------------------------------------------------------------------------------------------------------------
--//-- 10/100/1000 Mbps Ethernet -------
------------------------------------------------------------------------------------------------------------------------
ETHERNET_001: if (SIMULATION = '0') generate
	-- Interface with COM-5401 PHY#1 or COM-5102 PHY on the left connector. Also skip for faster simulations.
	
	-- MAC address automatically generated from unique DNA_ID
	LAN1_MAC_ADDR_GEN: process(CLK_P)
	begin
		if rising_edge(CLK_P) then
			if(DNA_ID_SAMPLE_CLK = '1') then
				LAN1_MAC_ADDR <= x"00" & DNA_ID_DATA(31 downto 0) & CREG(48)(7 downto 1) & "0";	-- unique, unicast  
				LAN2_MAC_ADDR <= x"00" & DNA_ID_DATA(31 downto 0) & CREG(48)(7 downto 1) & "1";	-- unique, unicast  
			end if;
		end if;
	end process;

    CLK_SKEW1 <= x"0310";   -- seems optimum for this board
    
	-- TODO: in the project, add a link to pre-synthesized COM5401.NGC component
	-- Source code (com-5401_011m4) is not included in this template, only the synthesized component.
	LAN_001: COM5401 
	GENERIC MAP(
		MII_SEL => '0',   	-- select which MII interface: 0 = RGMII, 1 = GMII/MII
		PHY_ADDR => "00111",	-- PHY_AD0/1 pulled-up by 1KOhm, PHY_AD2 pulled-up in .ucf file.
		CLK_FREQUENCY => CLK_FREQUENCY,	-- CLK_P in MHz, as time reference
		PLATFORM => 7,	-- Artix-7
		TARGET_PHY => 3, -- KSZ9031
		INTERNAL_PHY_CLKS_DELAYS => '0',
		SIMULATION => SIMULATION
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		MAC_TX_CONFIG => X"0003",	-- MAC must must provide pad + crc32
		MAC_RX_CONFIG => x"000F",	-- promiscuous mode, strip crc32, accept broadcast/multicast
		MAC_ADDR => LAN1_MAC_ADDR,
		PHY_CONFIG_CHANGE => CONFIG_CHANGE_PULSE_CLK_P,	
		PHY_RESET => '0',
		SPEED => "11",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		DUPLEX => '1',	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		TEST_MODE => "00",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		POWER_DOWN => '0', -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		CLK_SKEW1 => CLK_SKEW1, -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		CLK_SKEW2 => CLK_SKEW2, -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		MAC_TX_DATA => LAN1_MAC_TX_DATA,
		MAC_TX_DATA_VALID => LAN1_MAC_TX_DATA_VALID,
		MAC_TX_EOF => LAN1_MAC_TX_EOF,
		MAC_TX_CTS => LAN1_MAC_TX_CTS,
		MAC_RX_DATA => LAN1_MAC_RX_DATA,	
		MAC_RX_DATA_VALID => LAN1_MAC_RX_DATA_VALID,
		MAC_RX_SOF => LAN1_MAC_RX_SOF,
		MAC_RX_EOF => LAN1_MAC_RX_EOF,
		MAC_RX_CTS => '1',  -- follow-on processing is expected to always accept data even at max speed.
		-- RGMII interface ----------------
		TXC => LAN1_TXC,	
		TXD => LAN1_TXD,
		TX_CTL => LAN1_TX_CTL,
		RXC => LAN1_RXC,
		RXD => LAN1_RXD,
		RX_CTL => LAN1_RX_CTL,
		RESET_N => LAN1_RESET_N,
		MCLK => LAN1_MCLK,
		MDIO => LAN1_MDIO,	-- PHY#1
		-- end of RGMII interface --------
		MII_TX_CLK => '0',	-- MII interface is unused -----
		GMII_TX_CLK => open,
		GMII_MII_TXD => open,
		GMII_MII_TX_EN => open,
		GMII_MII_TX_ER => open,
		GMII_MII_CRS => '0',
		GMII_MII_COL => '0',
		GMII_MII_RX_CLK => '0',
		GMII_MII_RXD => x"00",
		GMII_MII_RX_DV => '0',
		GMII_MII_RX_ER => '0',  -- end of MII interface ------
		LINK_STATUS => LAN1_LINK_STATUS,
		SPEED_STATUS => LAN1_SPEED_STATUS,
		DUPLEX_STATUS => LAN1_DUPLEX_STATUS,
		PHY_ID => LAN1_PHY_ID,
		N_RX_FRAMES => open,
		N_RX_BAD_CRCS => LAN1_N_RX_BAD_CRCS,
		N_RX_FRAMES_TOO_SHORT => open,
		N_RX_FRAMES_TOO_LONG => open,
		N_RX_WRONG_ADDR => open,
		N_RX_LENGTH_ERRORS => open,
		CS_CLKG => open,
		CS1 => open,
		CS1_CLK => open,
		CS2 => open,
		CS2_CLK => open,
		CT1 => open,
		CT1_CLK => open,
		TP => COM5401_TP
	);

	LAN_002: COM5401 
	GENERIC MAP(
		MII_SEL => '0',   	-- select which MII interface: 0 = RGMII, 1 = GMII/MII
		PHY_ADDR => "00111",	-- PHY_AD0/1 pulled-up by 1KOhm, PHY_AD2 pulled-up in .ucf file.
		CLK_FREQUENCY => CLK_FREQUENCY,	-- CLK_P in MHz, as time reference
		PLATFORM => 7,	-- Artix-7
		TARGET_PHY => 3, -- KSZ9031
		INTERNAL_PHY_CLKS_DELAYS => '0',
		SIMULATION => SIMULATION
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		MAC_TX_CONFIG => X"0003",	-- MAC must must provide pad + crc32
		MAC_RX_CONFIG => x"000F",	-- promiscuous mode, strip crc32, accept broadcast/multicast
		MAC_ADDR => LAN2_MAC_ADDR,
		PHY_CONFIG_CHANGE => CONFIG_CHANGE_PULSE_CLK_P,	
		PHY_RESET => '0',
		SPEED => "11",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		DUPLEX => '1',	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		TEST_MODE => "00",	-- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		POWER_DOWN => '0', -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		CLK_SKEW1 => CLK_SKEW1, -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		CLK_SKEW2 => CLK_SKEW2, -- supersedes defaults within if PHY_CONFIG_CHANGE = '1'
		MAC_TX_DATA => LAN2_MAC_TX_DATA,
		MAC_TX_DATA_VALID => LAN2_MAC_TX_DATA_VALID,
		MAC_TX_EOF => LAN2_MAC_TX_EOF,
		MAC_TX_CTS => LAN2_MAC_TX_CTS,
		MAC_RX_DATA => LAN2_MAC_RX_DATA,	
		MAC_RX_DATA_VALID => LAN2_MAC_RX_DATA_VALID,
		MAC_RX_SOF => LAN2_MAC_RX_SOF,
		MAC_RX_EOF => LAN2_MAC_RX_EOF,
		MAC_RX_CTS => '1',  -- follow-on processing is expected to always accept data even at max speed.
		-- RGMII interface ----------------
		TXC => LAN2_TXC,	
		TXD => LAN2_TXD,
		TX_CTL => LAN2_TX_CTL,
		RXC => LAN2_RXC,
		RXD => LAN2_RXD,
		RX_CTL => LAN2_RX_CTL,
		RESET_N => LAN2_RESET_N,
		MCLK => LAN2_MCLK,
		MDIO => LAN2_MDIO,	-- PHY#1
		-- end of RGMII interface --------
		MII_TX_CLK => '0',	-- MII interface is unused -----
		GMII_TX_CLK => open,
		GMII_MII_TXD => open,
		GMII_MII_TX_EN => open,
		GMII_MII_TX_ER => open,
		GMII_MII_CRS => '0',
		GMII_MII_COL => '0',
		GMII_MII_RX_CLK => '0',
		GMII_MII_RXD => x"00",
		GMII_MII_RX_DV => '0',
		GMII_MII_RX_ER => '0',  -- end of MII interface ------
		LINK_STATUS => LAN2_LINK_STATUS,
		SPEED_STATUS => LAN2_SPEED_STATUS,
		DUPLEX_STATUS => LAN2_DUPLEX_STATUS,
		PHY_ID => LAN2_PHY_ID,
		N_RX_FRAMES => open,
		N_RX_BAD_CRCS => LAN2_N_RX_BAD_CRCS,
		N_RX_FRAMES_TOO_SHORT => open,
		N_RX_FRAMES_TOO_LONG => open,
		N_RX_WRONG_ADDR => open,
		N_RX_LENGTH_ERRORS => open,
		CS_CLKG => open,
		CS1 => open,
		CS1_CLK => open,
		CS2 => open,
		CS2_CLK => open,
		CT1 => open,
		CT1_CLK => open,
		TP => open
	);

	-- monitoring
	-- hardware test: read PHY_ID (LSB is sufficient)
	SREG(16) <= LAN1_PHY_ID(7 downto 0);
	SREG(17) <= LAN2_PHY_ID(7 downto 0);
	
	SREG(20) <= LAN1_MAC_ADDR(47 downto 40);
	SREG(21) <= LAN1_MAC_ADDR(39 downto 32);
	SREG(22) <= LAN1_MAC_ADDR(31 downto 24);
	SREG(23) <= LAN1_MAC_ADDR(23 downto 16);
	SREG(24) <= LAN1_MAC_ADDR(15 downto 8);
	SREG(25) <= LAN1_MAC_ADDR(7 downto 0);

--------------------------------------------------------------------------------------------------
-- REMOTE M&C  TCP SERVER
--------------------------------------------------------------------------------------------------
-- 1 UDP tx, 1 UDP rx port.
-- TCP server at port 1028, used for monitoring and control. Attached to LAN1

-- reclock with CLK_P for easier P&R
SERVER_CONTROL_RECLOCK_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		-- latch all at once, upon (re-)writing to the last control register
		if(CONFIG_CHANGE_PULSE_CLK_P = '1') then
			IP1_ADDRv4(31 downto 0) <= CREG(40) & CREG(41) & CREG(42) & CREG(43);	-- local IP address
			IP1_SUBNETMASK <= CREG(44) & CREG(45) & CREG(46) & CREG(47);	-- subnet mask
		end if;
		LAN1_GATEWAY <= x"00000000"; 		-- gateway IP address. n/a here. all local.
	end if;
end process;

	-- TODO: in the project, add a link to pre-synthesized COM5402.NGC component
	-- Source code (com-5402_007r) is not included in this template, only the synthesized component.
	TCP_SERVER_001: COM5402 
	GENERIC MAP(
		NUDPTX => 1,  	
		NUDPRX => 1,    
		IGMP_EN => '0',
		NTCPSTREAMS => NTCPSTREAMS,   -- 1
		CLK_FREQUENCY => CLK_FREQUENCY,	-- CLK_P in MHz, as time reference
		TX_IDLE_TIMEOUT => 50,    -- 50*4us = 200us 
		TCP_KEEPALIVE_PERIOD => 60,
		SIMULATION => SIMULATION
    )
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		MAC_ADDR => LAN1_MAC_ADDR,
		IPv4_ADDR => IP1_ADDRv4,
		IPv6_ADDR => x"00000000000000000000000000000000",
		MULTICAST_IP_ADDR => x"00000000",  -- no multicast
		SUBNET_MASK => IP1_SUBNETMASK,
		GATEWAY_IP_ADDR => LAN1_GATEWAY,
		CONNECTION_RESET => "0",
		TCP_KEEPALIVE_EN => '0',
		-- MAC interface
		MAC_TX_DATA => LAN1_MAC_TX_DATA,
		MAC_TX_DATA_VALID => LAN1_MAC_TX_DATA_VALID,
		MAC_TX_EOF => LAN1_MAC_TX_EOF,
		MAC_TX_SOF => LAN1_MAC_TX_SOF,
		MAC_TX_CTS => LAN1_MAC_TX_CTS,
		MAC_RX_DATA => LAN1_MAC_RX_DATA,
		MAC_RX_DATA_VALID => LAN1_MAC_RX_DATA_VALID,
		MAC_RX_SOF => LAN1_MAC_RX_SOF,
		MAC_RX_EOF => LAN1_MAC_RX_EOF,
		-- UDP rx
		UDP_RX_DATA => open,
		UDP_RX_DATA_VALID => open,
		UDP_RX_SOF => open,
		UDP_RX_EOF => open,
		UDP_RX_DEST_PORT_NO_IN => x"0000",
		CHECK_UDP_RX_DEST_PORT_NO => '0',	-- no verify destination port number in received UDP frames
		UDP_RX_DEST_PORT_NO_OUT => open,
		-- UDP tx									-- test test test for debugging with wireshark only
		UDP_TX_DATA => x"00",
		UDP_TX_DATA_VALID => '0',
		UDP_TX_SOF => '0',
		UDP_TX_EOF => '0',
		UDP_TX_CTS => open,
		UDP_TX_ACK => open,
		UDP_TX_NAK => open,
		UDP_TX_DEST_IP_ADDR => x"000000000000000000000000ac100146",
		UDP_TX_DEST_PORT_NO => x"0405",
		UDP_TX_SOURCE_PORT_NO => x"0406",
		-- TCP rx streams
		TCP_RX_DATA => LAN1_TCP_RX_DATA,
		TCP_RX_DATA_VALID => LAN1_TCP_RX_DATA_VALID,
		TCP_RX_RTS => LAN1_TCP_RX_RTS,
		TCP_RX_CTS => LAN1_TCP_RX_CTS,	
		-- TCP tx streams
		TCP_TX_DATA => LAN1_TCP_TX_DATA,
		TCP_TX_DATA_VALID => LAN1_TCP_TX_DATA_VALID,
		TCP_TX_CTS => LAN1_TCP_TX_CTS,	
		-- Monitoring
		TCP_CONNECTED_FLAG => TCP_CONNECTED_FLAG,
	--	CS1 => S1_1,
	--	CS1_CLK => S1_1_CLK,
	--	CS2 => S2_1,
	--	CS2_CLK => S2_1_CLK,
		TP => COM5402_TP	
	);

	-- one elastic buffer for each tcp rx stream
	-- M&C, TCP port 1028 (port is defined within COM5402 component)
	LAN1_TCP_RX_DATA0_X <= "0" & LAN1_TCP_RX_DATA(0);	
	ELASTIC_BUFFER_NRAMB_001: ELASTIC_BUFFER_NRAMB 
	generic map(
		NRAMB => 1,
		NBITS => 9,
		READ_AHEAD_1ADDRESS => '1'	-- no read latency, tricky timing ok because ARM is slower.
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		DATA_IN => LAN1_TCP_RX_DATA0_X,	-- 9 bit wide
		SAMPLE_CLK_IN => LAN1_TCP_RX_DATA_VALID(0),
		SAMPLE_CLK_IN_REQ => LAN1_TCP_RX_CTS(0),
		DATA_OUT => LAN_RX_DATA1_X,	-- 9 bit wide
		SAMPLE_CLK_OUT => open,
		SAMPLE_CLK_OUT_E => open,
		SAMPLE_CLK_OUT_REQ => RX_LAN_SAMPLE_CLK_REQ,
		BUFFER_EMPTY => RX_LAN_BUFFER_EMPTY
	);
	RX_LAN <= LAN_RX_DATA1_X(7 downto 0);
end generate;

-- TODO: instantiate another COM5402 component for the other LAN port, if needed
------------------------------------------------------------------------------------------------------------------------
--//-- TEST POINTS, LEDs
------------------------------------------------------------------------------------------------------------------------
-- TODO: define test points here
TP(12 downto 1) <= (others => '0');

-- LEDs ------------------
-- TODO: define LEDs here
LED(2) <= '1';		-- green power
LED(3) <= '1';		-- tx activity green
LED(4) <= '1';		-- rx activity green
LED(5) <= '1';		-- Transmitter ON 
LED(6) <= '1';		-- Receiver ON


-- example: extend any LED light pulse to at least 200ms to make it visible to the naked eye
--LED003_GEN: process(CLK_P)
--begin
--	if rising_edge(CLK_P) then
--		if(some pulse  = '1') then
--			LED_CNTR3 <= 2;
--			LED(3) <= '1';	
--		elsif(LED_CNTR3 = 0) then
--			LED(3) <= '0';	
--		elsif(TICK_100MS_rt = '1') then
--			-- decrement counter
--			LED_CNTR3 <= LED_CNTR3 -1;
--		end if;
--	end if;
--end process;

--// COMSCOPE2 -----------------------------------------------------
COMSCOPE_GEN: if (SIMULATION = '0') generate
	--  Comment in/out as needed. unused during simulation
	-- 
	-- TODO: Define Comscope traces here.
	-- Example: Trace 1
	S1_1 <= std_logic_vector(TEST_RAMP1(13 downto 6));	-- test transmit signal
	S1_1_CLK <= '1';
    
	-- Trace 2
	S2_1 <= std_logic_vector(TEST_RAMP2(13 downto 6));	-- test transmit signal
	S2_1_CLK <= '1';

	-- Trace 3

	-- Trigger (1-bit format, undefine

	COMSCOPE2_001: COMSCOPE2 
	generic map(
		TRACE1_ENABLED => '1',    -- enable as many simultaneous traces as needed
		TRACE2_ENABLED => '1',
		TRACE3_ENABLED => '1',
		TRACE4_ENABLED => '0',
		DATA_WIDTH1A => 8,
		ADDR_WIDTH1A => 9,
		ADDR_WIDTH1B => 9,
		DATA_WIDTH2A => 8,
		ADDR_WIDTH2A => 9,
		ADDR_WIDTH2B => 9,
		DATA_WIDTH3A => 8,
		ADDR_WIDTH3A => 9,
		ADDR_WIDTH3B => 9,
		DATA_WIDTH4A => 8,
		ADDR_WIDTH4A => 9,
		ADDR_WIDTH4B => 9
	)
	PORT MAP(
		CLK => CLK_RXg,
		SYNC_RESET => SYNC_RESET_CLK_RXg,
		REG238 => REG238,
		REG239 => x"00", --REG239,  acquire mode: sample
		REG240 => REG240,
		REG241 => REG241,
		REG242 => REG242,
		REG243 => REG243,
		REG244 => REG244,
		REG245 => REG245,
		REG246 => REG246,
		REG247 => REG247,
		REG248 => REG248,
		REG249 => REG249,
		S1_1 => S1_1,
		S1_1_VALID => S1_1_CLK,
		S1_2 => S1_2,
		S1_2_VALID => S1_2_CLK,
		S1_3 => S1_3,
		S1_3_VALID => S1_3_CLK,
		S1_4 => S1_4,
		S1_4_VALID => S1_4_CLK,
		S2_1 => S2_1,
		S2_1_VALID => S2_1_CLK,
		S2_2 => S2_2,
		S2_2_VALID => S2_2_CLK,
		S2_3 => S2_3,
		S2_3_VALID => S2_3_CLK,
		S2_4 => S2_4,
		S2_4_VALID => S2_4_CLK,
		S3_1 => S3_1,
		S3_1_VALID => S3_1_CLK,
		S3_2 => S3_2,
		S3_2_VALID => S3_2_CLK,
		S3_3 => S3_3,
		S3_3_VALID => S3_3_CLK,
		S3_4 => S3_4,
		S3_4_VALID => S3_4_CLK,
		S4_1 => (others => '0'),
		S4_1_VALID => '0',
		S4_2 => (others => '0'),
		S4_2_VALID => '0',
		S4_3 => (others => '0'),
		S4_3_VALID => '0',
		S4_4 => (others => '0'),
		S4_4_VALID => '0',
		T16b_1 => T16b_1,
		T16b_1_VALID => T16b_1_VALID,
		T16b_2 => T16b_2,
		T16b_2_VALID => T16b_2_VALID,
		T16b_3 => (others => '0'),
		T16b_3_VALID => '0',
		T16b_4 => (others => '0'),
		T16b_4_VALID => '0',
		SREG250 => SREG250,
		SREG251 => SREG251,
		TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
		FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
		SREG250_READ_TOGGLE => SREG250_READ_TOGGLE,
		START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE,
		TP => COMSCOPE2_TP
	);
end generate;


--// SIM2OUTFILE --------------------------------------------
-- simulation. send to a tab-delimited text file for subsequent (Matlab) plot.
--
--SIM2OUTFILE_X: if(SIMULATION = '1') generate
--	SIM2OUTFILE_001: SIM2OUTFILE 
--	generic map(
--		FILENAME => "tx_top1.txt"
--	)
--	PORT MAP(
--		CLK => CLK_TXg,
--		DATA1 => SN2_I(17 downto 6),	-- S+N
--		DATA2 => SN2_Q(17 downto 6),	
--		DATA3 =>  LOOPBACK_DATA1_I(13 downto 2), -- modulator output I,
--		SAMPLE_CLK_IN => '1'
--	);

--end generate;

--// ARM CO-PROCESSOR INTERFACE ---------------------------------------------------
-- Primarily used for monitoring and control.
-- 8-bit shared address/data bus. clock synchronous.
-- PART OF THE FRAMEWORK. DO NOT CHANGE unless you want to 
-- add more control registers (see UC_WRITE_001) or status registers (see UC_READ_001).
-- 
-- Address latch
UC_ADDR_001: process(UC_WEBG)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '1') then
			UC_ADDRESS <= unsigned(UC_AD);
	  	end if;
  	end if;
end process;

-- Read status registers 
-- This template defines 10 status registers. 
-- User can define additional control registers using the same pattern. 
UC_READ_001: process(UC_ADDRESS, REG255, RX_LAN, SREG251, SREG250, SREG)
begin
	case(UC_ADDRESS) is
		-- ComBlock-reserved addresses ----
		when "11111111" => UC_AD_local <= REG255;		-- reg 255. Async serial from 3 connectors
		when "11111110" => UC_AD_local <= RX_LAN;		-- reg 254 = 8-bit data from LAN M&C channel
		when "11111101" => UC_AD_local <= REVISION;	-- reg 253 = fpga revision
		when "11111100" => UC_AD_local <= OPTION;		-- reg 252 = fpga option
		when "11111011" => UC_AD_LOCAL <= SREG251;		-- reg 251 = comscope  status
		when "11111010" => UC_AD_LOCAL <= SREG250;		-- reg 250 = comscope data byte
		
		-- User-defined status registers -------
		-- Note: SREG0-15 are inserted by the ARM co-processor, not the FPGA.
		when "00010000" => UC_AD_local <= SREG(16);
		when "00010001" => UC_AD_local <= SREG(17);
		when "00010010" => UC_AD_local <= SREG(18);
		when "00010011" => UC_AD_local <= SREG(19);
		when "00010100" => UC_AD_local <= SREG(20);
		when "00010101" => UC_AD_local <= SREG(21);
		when "00010110" => UC_AD_local <= SREG(22);
		when "00010111" => UC_AD_local <= SREG(23);
		when "00011000" => UC_AD_local <= SREG(24);
		when "00011001" => UC_AD_local <= SREG(25);
-- TODO: in above code, define your own status registers
--		when "00011010" => UC_AD_local <= SREG(26);
--		when "00011011" => UC_AD_local <= SREG(27);
--		when "00011100" => UC_AD_local <= SREG(28);
--		when "00011101" => UC_AD_local <= SREG(29);
--		when "00011110" => UC_AD_local <= SREG(30);
--		when "00011111" => UC_AD_local <= SREG(31);
--		when "00100000" => UC_AD_local <= SREG(32);
--		when "00100001" => UC_AD_local <= SREG(33);
--		when "00100010" => UC_AD_local <= SREG(34);
--		when "00100011" => UC_AD_local <= SREG(35);
--		when "00100100" => UC_AD_local <= SREG(36);
--		when "00100101" => UC_AD_local <= SREG(37);
--		when "00100110" => UC_AD_local <= SREG(38);
--		when "00100111" => UC_AD_local <= SREG(39);
--		when "00101000" => UC_AD_local <= SREG(40);
--		when "00101001" => UC_AD_local <= SREG(41);
--		when "00101010" => UC_AD_local <= SREG(42);
--		when "00101011" => UC_AD_local <= SREG(43);
--		when "00101100" => UC_AD_local <= SREG(44);
--		when "00101101" => UC_AD_local <= SREG(45);
--		when "00101110" => UC_AD_local <= SREG(46);
--		when "00101111" => UC_AD_local <= SREG(47);
--		when "00110000" => UC_AD_local <= SREG(48);
--		when "00110001" => UC_AD_local <= SREG(49);
--		when "00110010" => UC_AD_local <= SREG(50);
--		when "00110011" => UC_AD_local <= SREG(51);
--		when "00110100" => UC_AD_local <= SREG(52);
--		when "00110101" => UC_AD_local <= SREG(53);
--		when "00110110" => UC_AD_local <= SREG(54);
--		when "00110111" => UC_AD_local <= SREG(55);
--		when "00111000" => UC_AD_local <= SREG(56);
--		when "00111001" => UC_AD_local <= SREG(57);
--		when "00111010" => UC_AD_local <= SREG(58);
--		when "00111011" => UC_AD_local <= SREG(59);
		when others => UC_AD_local <= x"00";

	end case;
end process;


UC_AD <= UC_AD_local when (UC_CSIB_IN = '0') and (UC_REB_IN = '0') else (others => 'Z');
	-- when to drive the data bus between ARM and FPGA
	
-- toggle a few flags upon reading specific status registers
UC_READ_002: process(ASYNC_RESET, UC_REBG)
begin
	if(ASYNC_RESET = '1') then
		SREG16_READ_TOGGLE <= '0';
		SREG250_READ_TOGGLE <= '0';
		SREG254_READ_TOGGLE <= '0';
	elsif rising_edge(UC_REBG) then
		if (UC_CSIB_IN = '0')  then
		
			if (UC_ADDRESS = 16) then
				SREG16_READ_TOGGLE <= not SREG16_READ_TOGGLE;
			end if;
			if (UC_ADDRESS = 250) then
				SREG250_READ_TOGGLE <= not SREG250_READ_TOGGLE;
			end if;
			if (UC_ADDRESS = 254) then
				SREG254_READ_TOGGLE <= not SREG254_READ_TOGGLE;
			end if;
			
			
		end if;
	end if;
end process;

UC_READ_003a: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		SREG16_READ_TOGGLE1_D <= SREG16_READ_TOGGLE;
		SREG16_READ_TOGGLE1_D2 <= SREG16_READ_TOGGLE1_D;
		
		if(SREG16_READ_TOGGLE1_D /= SREG16_READ_TOGGLE1_D2) then
			-- latch multi-byte value upon reading SREG16
			LATCH_MONITORING_PULSE <= '1';
		else
			LATCH_MONITORING_PULSE <= '0';
		end if;
	end if;
end process;

UC_READ_003b: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		SREG16_READ_TOGGLE2_D <= SREG16_READ_TOGGLE;
		SREG16_READ_TOGGLE2_D2 <= SREG16_READ_TOGGLE2_D;
		
		if(SREG16_READ_TOGGLE2_D /= SREG16_READ_TOGGLE2_D2) then
			-- latch multi-byte value upon reading SREG16
			LATCH_MONITORING_PULSE_CLK_RX <= '1';
		else
			LATCH_MONITORING_PULSE_CLK_RX <= '0';
		end if;
	end if;
end process;

UC_READ_003c: process(CLK_TXg)
begin
	if rising_edge(CLK_TXg) then
		SREG16_READ_TOGGLE3_D <= SREG16_READ_TOGGLE;
		SREG16_READ_TOGGLE3_D2 <= SREG16_READ_TOGGLE3_D;
		
		if(SREG16_READ_TOGGLE3_D /= SREG16_READ_TOGGLE3_D2) then
			-- latch multi-byte value upon reading SREG16
			LATCH_MONITORING_PULSE_CLK_TX <= '1';
		else
			LATCH_MONITORING_PULSE_CLK_TX <= '0';
		end if;
	end if;
end process;

-- write to FPGA control registers
UC_WRITE_001ax: if (SIMULATION = '0') generate
	UC_WRITE_001a: process(UC_WEBG)
	begin
		if rising_edge(UC_WEBG) then
			if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
				-- registers REG0-41 reserved for ARM processor 
				for I in NCREG_IMIN to NCREG_IMAX loop
					if(UC_ADDRESS = I) then
						CREG(I) <= UC_AD;
					end if;
				end loop;
			end if;
		end if;
	end process;
end generate;

UC_WRITE_001b: process(UC_WEBG)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			if (UC_ADDRESS = 200) then
				REG200 <= UC_AD;	-- RF synthesizers (controlled from ARM micro)
			end if;
			if (UC_ADDRESS = 201) then
				REG201 <= UC_AD;	-- RF gain LSB
			end if;
			if (UC_ADDRESS = 202) then
				REG202 <= UC_AD;	-- RF gain MSB
				RF_AGC_LEVEL_CHANGE_TOGGLE <= not RF_AGC_LEVEL_CHANGE_TOGGLE;
			end if;
			if (UC_ADDRESS = 203) then
				REG203 <= UC_AD;	-- IF gain LSB
			end if;
			if (UC_ADDRESS = 204) then
				REG204 <= UC_AD;	-- IF gain MSB
				IF_AGC_LEVEL_CHANGE_TOGGLE <= not IF_AGC_LEVEL_CHANGE_TOGGLE;
			end if;


			if (UC_ADDRESS = NCREG_IMAX) then
				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- Configuration change toggles upon writing the last control register
				-- This happens invisibly to the user (only the ARM uC writes to the last control register
				-- which is invible to the user and to the ComBlock Control Center).
				UC_CONFIGURATION_DONE <= '1';
			end if;


			if (UC_ADDRESS = 68) then
				MOD_TX_LEVEL_CHANGE_TOGGLE <= not MOD_TX_LEVEL_CHANGE_TOGGLE;	
				-- tx level change
			end if;

			-- Reserved:
			-- REG254 reserved for output serial
			-- REG255 write reserved for output serial (8-bit parallel format)
			if (UC_ADDRESS = 255) then
				-- REG255 write reserved for output serial (8-bit parallel format)
				-- route response to host query over LAN/TCP-IP
				LAN1_TCP_TX_DATA(0) <= UC_AD;
				TX_LAN_TOGGLE <= not TX_LAN_TOGGLE;
			end if;
		end if;
	end if;
end process;

CREG_SIM: if(SIMULATION = '1') generate
-- initialize control registers for simulation
   CREG(39) <= x"0f";	-- enable 10 MHz ports, enable tx synthesizer
	CREG(40) <= x"c0";   -- local IP address (MSB)
	CREG(41) <= x"a8";   --
	CREG(42) <= x"01";   --
	CREG(43) <= x"02";   -- local IP address (LSB)
	CREG(44) <= x"FF";   -- subnet mask (LSB)
	CREG(45) <= x"FF";   -- 
	CREG(46) <= x"FF";   -- 
	CREG(47) <= x"ff";   -- subnet mask (MSB)
	CREG(48) <= x"ff";   -- MAC address LSB
end generate;


--// reserved COMSCOPE -------------------------------------------------------
-- Comment out if ComScope feature is unused
UC_WRITE_003: process(UC_WEBG)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			-- Reserved for ComScope			
--			if (UC_ADDRESS = 237) then
--				REG237 <= UC_AD;	
--			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

--// end of COMSCOPE section -------------------------------------------------------

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_P
RECLOCK_CLK_P_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		CONFIG_CHANGE_TOGGLE1_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE1_D2 <= CONFIG_CHANGE_TOGGLE1_D;
		CONFIG_CHANGE_PULSE_CLK_P_D <= CONFIG_CHANGE_PULSE_CLK_P;
		if(CONFIG_CHANGE_TOGGLE1_D /= CONFIG_CHANGE_TOGGLE1_D2) then
			CONFIG_CHANGE_PULSE_CLK_P <= '1';
		else
			CONFIG_CHANGE_PULSE_CLK_P <= '0';
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_RXg
RECLOCK_CLK_RX_001: process(CLK_RXg)
begin
	if rising_edge(CLK_RXg) then
		CONFIG_CHANGE_TOGGLE2_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE2_D2 <= CONFIG_CHANGE_TOGGLE2_D;
		if(CONFIG_CHANGE_TOGGLE2_D /= CONFIG_CHANGE_TOGGLE2_D2) then
			CONFIG_CHANGE_PULSE_CLK_RXg <= '1';
		else
			CONFIG_CHANGE_PULSE_CLK_RXg <= '0';
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_TXg
RECLOCK_CLK_TXg_001: process(CLK_TXg)
begin
	if rising_edge(CLK_TXg) then
		CONFIG_CHANGE_TOGGLE4_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE4_D2 <= CONFIG_CHANGE_TOGGLE4_D;
		CONFIG_CHANGE_PULSE_CLK_TXg_D <= CONFIG_CHANGE_PULSE_CLK_TXg;
		
		if(CONFIG_CHANGE_TOGGLE4_D /= CONFIG_CHANGE_TOGGLE4_D2) then
			CONFIG_CHANGE_PULSE_CLK_TXg <= '1';
		else
			CONFIG_CHANGE_PULSE_CLK_TXg <= '0';
		end if;
	end if;
end process;

RECLOCK_CLK_P_002: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		MOD_TX_LEVEL_CHANGE_TOGGLE_D <= MOD_TX_LEVEL_CHANGE_TOGGLE;
		MOD_TX_LEVEL_CHANGE_TOGGLE_D2 <= MOD_TX_LEVEL_CHANGE_TOGGLE_D;
		if(MOD_TX_LEVEL_CHANGE_TOGGLE_D /= MOD_TX_LEVEL_CHANGE_TOGGLE_D2) then
			MOD_TX_LEVEL_CHANGE_PULSE <= '1';
		else
			MOD_TX_LEVEL_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;


-- DNA Port ------------------------------------
-- Read FPGA unique DNA ID
DNA_ID_001: DNA_ID 
	GENERIC MAP(
		CLK_FREQUENCY => CLK_FREQUENCY	-- CLK_P in MHz, as time reference
	)
	PORT MAP(
		CLK => CLK_P,
		SYNC_RESET => SYNC_RESET_CLK_P,
		ID_OUT => DNA_ID_DATA,
		ID_VALID_OUT => open,
		ID_SAMPLE_CLK_OUT => DNA_ID_SAMPLE_CLK 
);

-- M&C LAN interface --------------------------
-- REG255 reserved for reading serial port inputs
-- These signals are pulled high (to mimic stop bit).
REG255 <= "1111" & RX_LAN_BUFFER_EMPTY & "111";

	-- indicates AVAILABILITY of 8-bit byte from LAN. Not actual data.
	-- When low, uC must read byte from LAN

-- Reclock TX_LAN_TOGGLE
RECLOCK_TX_LAN_TOGGLE: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		TX_LAN_TOGGLE_D <= TX_LAN_TOGGLE;
		TX_LAN_TOGGLE_D2 <= TX_LAN_TOGGLE_D;
		if(TX_LAN_TOGGLE_D /= TX_LAN_TOGGLE_D2) then
			LAN1_TCP_TX_DATA_VALID(0) <= '1';
		else
			LAN1_TCP_TX_DATA_VALID(0) <= '0';
		end if;
	end if;
end process;

-- Immediately upon completing reading a M&C byte from the LAN, ask for the 
-- next one. 
-- RACE CONDITION??????? ARM IS FAST!!!!
RECLOCK_RDN_D: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		SREG254_READ_TOGGLE_D <= SREG254_READ_TOGGLE;
		SREG254_READ_TOGGLE_D2 <= SREG254_READ_TOGGLE_D;

		RX_LAN_BUFFER_EMPTY_D <= RX_LAN_BUFFER_EMPTY;
		RX_LAN_BUFFER_EMPTY_D2 <= RX_LAN_BUFFER_EMPTY_D;


		if (SREG254_READ_TOGGLE_D2 /= SREG254_READ_TOGGLE_D) and (RX_LAN_BUFFER_EMPTY_D2 = '0') then
			-- end of M&C byte read from ARM uC.
			-- last M&C byte read was over LAN. Increment LAN elastic buffer read pointer
			RX_LAN_SAMPLE_CLK_REQ <= '1';
		else
			RX_LAN_SAMPLE_CLK_REQ <= '0';
		end if;
	end if;
end process;

end Behavioral;
