-------------------------------------------------------------
--	Filename:  SPI_3PINS.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 1
--	Date last modified: 5/12/13
-- Inheritance: 	SPI.vhd 2/10/04  com-1202
--
-- Description: Serial port interface to read/write registers 
-- from/to AD970X TX DAC
-- 3-pins SPI (SDIO is shared for tx and rx data, no SDO). Use SPI_4pins alternate code instead when dedicated SDO is available.
-- Supports 8-bit read/write.
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SPI is 
generic (
   NDIV: integer range 2 to 256:= 2
	-- divide master clock by this integer to slow down the interface with the SPI device.
	-- must be EVEN (for equally spaced rising and falling edges)
	-- AD970x: 20 MHz max SPI clk
	-- AD9634: 25 MHz max SPI clk
);	
port ( 

	--// Clocks Resets
   CLK : in std_logic;				-- Master clock for this FPGA
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high
		
	--// control
	ADDR: in std_logic_vector(4 downto 0);
		-- This information can change immediately after the request is made
	DATA_OUT: in std_logic_vector(7 downto 0);
		-- byte to be written to the specified register over SPI
		-- This information can change immediately after the request is made
	WRITE_REQ: in std_logic;
		-- 1 CLK pulse indicating that the DATA_OUT is to be written out
		-- to the specified register.
	READ_REQ: in std_logic;
		-- 1 CLK pulse indicating a read request
	DATA_IN: out std_logic_vector(7 downto 0);
		-- byte read from the specified register over SPI
	DATA_IN_CLK: out std_logic;
		-- 1 CLK pulse indicating that DATA_IN is valid.
	BUSY: out std_logic;
		-- '0' to indicate that the SPI interface is available. Goes to '1'
		-- immediately after WRITE_REQ and READ_REQ until command is complete.

  --// SPI interface
  CSB: out std_logic := '1';	-- chip select#, active low
  SCLK: out std_logic := '0';
  SDIO: inout std_logic := '0'

);
end entity;

architecture behavioral of SPI is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal NDIV_CNTR: integer range 0 to 256 := 0;
signal CLK_DIV: std_logic := '0';
signal CS: std_logic := '0';
signal WRITE_REQ_FLAG: std_logic := '0';
signal READ_REQ_FLAG: std_logic := '0';
signal CLK_COUNTER: std_logic_vector(5 downto 0) := (others => '0');
signal WORD_OUT:std_logic_vector(15 downto 0) := (others => '0');
signal NEXT_WORD_OUT:std_logic_vector(15 downto 0) := (others => '0');
signal RWN: std_logic := '0';
signal SDIO_DIR: std_logic := '0';
signal DATA_IN_local: std_logic_vector(7 downto 0)  := (others => '0');

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin

-- first, slow down the SPI clock. Modulo NDIV counter.
SLOWER_CLK_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(NDIV_CNTR = 0) then
			NDIV_CNTR <= NDIV-1;
			CLK_DIV <= '1';
		else
			NDIV_CNTR <= NDIV_CNTR - 1;
			if(NDIV_CNTR = NDIV/2) then
				CLK_DIV <= '1';
			else
				CLK_DIV <= '0';
			end if;
		end if;
	end if;
end process;

-- hold the write or read request until next CLK_DIV
HOLD_WRITE_REQUEST_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(WRITE_REQ = '1') then
			WRITE_REQ_FLAG <= '1';
		elsif(CLK_DIV = '1') and (CLK_COUNTER = 0) and (WRITE_REQ_FLAG = '1') then
			WRITE_REQ_FLAG <= '0';
		end if;
	end if;
end process;

HOLD_READ_REQUEST_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(READ_REQ = '1') then
			READ_REQ_FLAG <= '1';
		elsif(CLK_DIV = '1') and (CLK_COUNTER = 0) and (WRITE_REQ_FLAG = '0')  and (READ_REQ_FLAG = '1') then
			READ_REQ_FLAG <= '0';
		end if;
	end if;
end process;



--// The SPI waveform consists of 16 clock cycles, or 34 phases
-- Idle state when CLK_COUNTER = 0.
CLK_COUNTER_GEN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than TBD MHz (see device specifications)
			if(CLK_COUNTER = 0) and (WRITE_REQ_FLAG = '1') then
				-- start write transaction
				RWN <= '0';
			 	-- start counting
				CLK_COUNTER <= "000001";
				CS <= '1';	-- chip select
			elsif(CLK_COUNTER = 0) and (READ_REQ_FLAG = '1') then
				-- start read transaction
				RWN <= '1';
			 	-- start counting
				CLK_COUNTER <= "000001";
				CS <= '1';	-- chip select
			elsif(CLK_COUNTER = 34) then
				-- transaction complete
				CLK_COUNTER <= (others => '0');
				CS <= '0';
				SDIO_DIR <= '0';	-- switch back SDIO direction to output
		  	elsif(CLK_COUNTER /= 0) then
				-- once the cycle is started, finish it.
				CLK_COUNTER <= CLK_COUNTER + 1;
				if(CLK_COUNTER = 16) and (RWN = '1') then
					-- time to switch direction on the bidirectional SDIO line to input
					SDIO_DIR <= '1';
				end if;
			end if;
		end if;
	end if;
end process;
SCLK <= not CLK_COUNTER(0);
CSB <= not CS;

-- unavailable for another read/write request until end of cycle
BUSY <= '1' when (WRITE_REQ = '1') or (READ_REQ = '1') or  (WRITE_REQ_FLAG = '1') or (READ_REQ_FLAG = '1') or 
		(CLK_COUNTER /= 0) else '0';

-- latch in the write data and shift it out one bit at a time
DATA_OUT_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(WRITE_REQ = '1') and (WRITE_REQ_FLAG = '0') then
			-- save address/data until the state machine is available
			NEXT_WORD_OUT <= "000" & ADDR & DATA_OUT;
		elsif(READ_REQ = '1') and (READ_REQ_FLAG = '0') then
			-- save address until the state machine is available
			NEXT_WORD_OUT <= "100" & ADDR & x"00";
		elsif(CLK_DIV = '1') and (CLK_COUNTER = 0) and ((WRITE_REQ_FLAG = '1') or (READ_REQ_FLAG = '1')) then
			-- start transaction
			WORD_OUT <= NEXT_WORD_OUT;
		elsif (CLK_DIV = '1') and (CLK_COUNTER(0) = '0') then
			-- shift left
			WORD_OUT(WORD_OUT'left downto 1) <= WORD_OUT((WORD_OUT'left - 1) downto 0);
		end if;
	end if;
end process;
SDIO <= WORD_OUT(WORD_OUT'left) when (SDIO_DIR = '0') else 'Z';

DATA_IN_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') and (SDIO_DIR = '1') and (CLK_COUNTER(0) = '1') then
			-- shift in new received data bit
			DATA_IN_local(7 downto 0) <= DATA_IN_local(6 downto 0) & SDIO;
		elsif  (CLK_DIV = '1') and (CLK_COUNTER = 32) then
			-- latch read byte out
			DATA_IN <= DATA_IN_local;
			DATA_IN_CLK <= '1';
		else
			DATA_IN_CLK <= '0';
		end if;
	end if;
end process;


end behavioral;